/*********************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************/

#include <xc.h>
#include <stdlib.h>
#include <time.h>

#define FOSC                     32000000UL
// Timer0: 1:32 prescale
#define OPTION_REG_INIT          0b10000101
#define TIMER0_PRESCALE          64L

#include "GenericTypeDefs.h"
#include "lcd.h"
#include "rtcc.h"
#include "input.h"
#include "i2c.h"
#include "mcp9800.h"
#include "mchp_support.h"

void display_time(void);
void display_temp(void);
char display_int(int t);
void display_pot(int p);
BOOL TimeBaseManager(void);
void acquire_display_data(void);
void acquire_temperature(BOOL);
void acquire_time(void);
void Update_LCD(void);

volatile char          blink;
bit                    TMR0_time_tic;
bit                    TMR0_button_flag;
volatile unsigned char timebase_10ms;
volatile unsigned char TMR0_button_held_timer;
volatile unsigned char TMR0_temperature_timer;

typedef enum { MODE_TIME, MODE_TEMPERATURE, MODE_POT} mode_t;
typedef enum { INCR_MINUTES, INCR_TENS, INCR_HOURS } mode_t_incr;

int         pot_value;
time_t      time_out;
event_t     btn_event;
char        time_set;
mode_t      display_mode;
mode_t_incr time_incr_mode;
BOOL        am;
BOOL        time_dp;
BCD_TYPE    bcd_data;

int main(void)
{
	time_t      t;
	
	blink = 0;

	OPTION_REG = OPTION_REG_INIT;	
	OSCCONbits.IRCF = 14;
	SPLLEN = 1;
		    
	// peripheral configuration
	PORTC = 0;
	PORTD = 0;
	PORTE = 0;
	TRISC2 = 0;
	TRISE0 = 0;
	TRISE1 = 0;
	TRISE2 = 0;
	TRISD1 = 0;
	TRISD5 = 0;
	TRISD6 = 0;
	
	ANSA1 = 1;
	ANSA3 = 1;
	ANSB3 = 1;
	ANSB1 = 1;
	
	display_mode = MODE_TIME;    
	//time_set = 0;
	    
	rtcc_init();
	lcd_init();
	input_init();
	    
	i2c_init();
	t = time(0);
	
	LCDIE = 1;
	PEIE = 1;
	GIE = 1;
	
	// external HW configuration
	mcp9800_init();
    
	// light all segments
	AMPS = 1;
	VOLT = 1;
	KILO = 1;
	OHMS = 1;
	S1 = 1;
	S2 = 1;
	MILLI = 1;
	MEGA = 1;
	B1 = G1 = C1 = 1;
	A1 = F1 = E1 = D1 = 1;
	B2 = G2 = C2 = DP2 = 1;
	D2 = E2 = F2 = A2 = 1;
	DP3 = C3 = G3 = B3 = 1;
	D3 = E3 = F3 = A3 = 1;
	DP4 = BC4 = RH = DH = 1;
	AC = MINUS = BATT = RC = 1;
	// wait for 2 seconds
	for(time_out = t+2; time_out > time(0); ){CLRWDT();}     
   
	// wait for LCD ready
	while(!WA);
	// clear all segments
	AMPS = 0;
	VOLT = 0;
	KILO = 0;
	OHMS = 0;
	S1 = 0;
	S2 = 0;
	MILLI = 0;
	MEGA = 0;
	B1 = G1 = C1 = 0;
	A1 = F1 = E1 = D1 = 0;
	B2 = G2 = C2 = DP2 = 0;
	D2 = E2 = F2 = A2 = 0;
	DP3 = C3 = G3 = B3 = 0;
	D3 = E3 = F3 = A3 = 0;
	DP4 = BC4 = RH = DH = 0;
	AC = MINUS = BATT = RC = 0;
   
   ////////////////////////////////////////// main loop ////////////////////////////////////////////////   
   while(1)
   {  
        CLRWDT();
		while (!TimeBaseManager());
		pot_value = input_pot(); 	
		btn_event = input_event();
        
		switch(btn_event)
		{
			case BUTTON_UP: 
				break;
			case BUTTON_DOWN:
				if(display_mode == MODE_TIME && time_set == 0)
				{
					if(TMR0_button_flag)
						if(TMR0_button_held_timer-- == 0)
						{
							// enter time set mode when the button is held long enough
							time_set = 1;
							// time set routine will decrement TMR0_button_held_timer to zero
							// and make the first time adjustment immediately
							TMR0_button_held_timer++;
							// read the current time into t and strip off seconds
							time(&t);
							t = t - (t % 60);
							time_incr_mode = INCR_MINUTES;
						}
						else
							TMR0_button_flag = 0;
				}	
				break;
			case BUTTON_PRESSED:
				if(display_mode == MODE_TIME)
				{	// set the held-button timer for 0.5 sec (0.5/Timebase manager resolution)
					// button must be held down for this amount of time to enter time set mode
					TMR0_button_held_timer = 50;
				}    
				break;
			case BUTTON_RELEASED: // on release, next display mode (unless we were setting the time)
				if(time_set == 0)
				{
					switch(display_mode)
					{
						case MODE_TIME:
							TMR0_temperature_timer = 1;
							display_mode = MODE_TEMPERATURE;
							break;
						case MODE_TEMPERATURE:
							display_mode = MODE_POT;
							break;
						default:
						case MODE_POT:
							display_mode = MODE_TIME; 
							break;
					}
				}
				else
					// exit time set mode on button release
					// time resumes from where it was left
					time_set = 0;
				break;
			default: break;
		}
		
		// Time set increments the clock time by one minute for every held button interval
		// when the minutes reach an even 10 then the increment is 10 minutes
		// when the minutes reach an even hour then the increment is one hour
		if(time_set) // if we are setting the time ...
		{
			if(TMR0_time_tic)
			{
				TMR0_time_tic = 0;
				if(TMR0_button_held_timer-- == 0)
					{
						// increments occur every 1/3 second while the button is held
						TMR0_button_held_timer =  33;
						// adjust the time
						switch (time_incr_mode)
						{
							case INCR_MINUTES:
								t += 60;             // increment minutes while not even 10
								if ((t % 600)==0)    // check even 10
									time_incr_mode = INCR_TENS;
								break;
							case INCR_TENS:
								t += 600;             // increment by 10 minutes while not even hour
								if ((t % 3600)==0)    // check even 60
									time_incr_mode = INCR_HOURS;
								break;
							case INCR_HOURS:
							default:
								t += 3600;  // increment hour
								break;
					} // end - switch time_incr_mode
				} // end - button held timer
				// show the current setting
				rtcc_set(&t);
			} // end - TMR0_time_tic
		} // end - time set
		
		acquire_display_data();
		Update_LCD();
		// run and loop time indicator
		LATD1 = !LATD1;
	} // end - main loop  	    
	return 0;
}

void interrupt isr()
{
	static bit icon;
    // Timer 1
    if(TMR1IF && TMR1IE)
    {
        TMR1IF = 0;
        rtcc_handler();
        icon = !icon;
        //S2 = ~S2;

        if(blink && RE0)
            lcd_display_off();
        else
            lcd_display_on();    
            
    }

    if(LCDIF && LCDIE)
    {
	    LCDIF = 0;
	    if(icon)
	    	S2 = 1;
	    else
	    	S2 = 0;
	}
	
    i2c_handler();
}

void display_temp(void)
{
    // temperature is assumed to have been acquired, converted, and stored in global bcd_data 

    lcd_display_digits(bcd_data);
    DP3 = 0;
    DP2 = 1;
    S1 = 0;
    S2 = 0;
    AMPS = 0;
    VOLT = 0;
    KILO = 1;
    OHMS = 0;
    MINUS = 0;
    DH = 0;
    RH = 0;
    RC = 0;
}
    
void display_pot(int p)
{
    display_int(p);
    DP3 = 0;
    DP2 = 0;
    S1 = 0;
    S2 = 1;
    AMPS = 0;
    VOLT = 0;
    KILO = 0;
    OHMS = 0;
    MINUS = 0;
    DH = 1;
    RH = 0;
    RC = 0;
}

char display_int(int t)
{
    BCD_TYPE bcd;
    
    // assumes t is the temperature in degrees C * 10
    bcd.digit0 = t %10;
    t /= 10;
    bcd.digit1 = t % 10;
    t /= 10;
    bcd.digit2 = t % 10;
    t /= 10;
    bcd.digit3 = t%10;
    return(lcd_display_digits(bcd));   
}    

void display_time(void)
{
    // display time in 12 hour format
    // time is assumed to have been acquired, converted, and stored in global bcd_data
    
    lcd_display_digits(bcd_data);
    if(time_dp)
       DP3 = 1;
    else
       DP3 = 0;

    if(am)
      AMPS = 1;
    else
      AMPS = 0;
          
    DP2 = 0;
    S1 = 0;
    //AMPS = 0;
    VOLT = 0;
    KILO = 0;
    OHMS = 0;
    RC = 1;
    DH = 0;
    RH = 0;
}

//////////////////////////////////////// definitions for TimeBaseManager //////////////////////////////////////////

#define SYSTEM_FREQUENCY                  (FOSC/4L)
#define MILLISECONDS_PER_SEC              1000L

// number of milliseconds in each timebase count
// (i.e. resolution of TimebaseManager() in milliseconds)
#define TIMEBASE_MS_PER_COUNT             10L

// number of milliseconds in each timebase manager tick clock
#define TIMEBASE_MS_PER_TICK              2L
#define TIMEBASE_LOAD_10ms                (TIMEBASE_MS_PER_COUNT/TIMEBASE_MS_PER_TICK)

// timebase reload value for TimebaseManager() interrupt period: 
#define TIMEBASE_MANAGER_PERIOD_COUNT     (TIMEBASE_MS_PER_TICK*SYSTEM_FREQUENCY)/(TIMER0_PRESCALE*MILLISECONDS_PER_SEC)

#define TIMEBASE_MANAGER_RELOAD_COUNT     (0x00FF - TIMEBASE_MANAGER_PERIOD_COUNT)

#if (TIMEBASE_MANAGER_PERIOD_COUNT > 0x00FF)
   #error "TIMER0_PRESCALE is too small. Timebase reload count exceeds 0xFF." 
#endif

#if (TIMEBASE_MANAGER_PERIOD_COUNT < 0x80)
   #warning "TIMER0_PRESCALE can be decreased to improve timebase resolution"
#endif

BOOL TimeBaseManager(void) 
{  
	if(T0IF) 
   {
		T0IF = 0;
		TMR0 += TIMEBASE_MANAGER_RELOAD_COUNT;

		if(timebase_10ms) {
			timebase_10ms--;
			return TRUE;								// wait 10ms
		}

		// 10ms TIMEBASE timer
		//--------------------------------------------------------------------------
		timebase_10ms = TIMEBASE_LOAD_10ms;

      // For regulating main loop time
      TMR0_time_tic = 1;
      // For timing held buttons
      TMR0_button_flag = 1;
      
      return TRUE;
   }
   return FALSE;
}

void Update_LCD(void)
{
    // call this routine upon sensing WA rising edge
    switch(display_mode)
    {
    case MODE_TIME: 
        display_time();
        break;
    case MODE_TEMPERATURE:
        display_temp();
        break;
    case MODE_POT:
        display_pot(pot_value);
        break;
    default:
        break;
    }
}

void acquire_time(void)
{
    // acquire the present time and convert to BCD digits. Store result in global bcd_data

    time_t t;
    struct tm *d;
    
    time(&t);
    d = gmtime(&t);
    
    // time_dp is blinking seconds indicator
    time_dp = ((t % 2) == 1);

    bcd_data.digit0 = d->tm_min % 10;
    bcd_data.digit1 = d->tm_min / 10;
    
    if(d->tm_hour > 11)
    {
        d->tm_hour -= 12;
        am = FALSE;  // PM indicator
    }    
    else
    {
        am = TRUE;   // AM indicator
    }
      
    if(d->tm_hour == 0) d->tm_hour = 12;
    
    bcd_data.digit2 = d->tm_hour % 10;
    bcd_data.digit3 = d->tm_hour/10;
}

void acquire_temperature(BOOL ModeF)
{
    // acquire temperature from remote device and convert and store in global bcd_data
    int deg;
    
    deg = mcp9800_get_temp();
    // assumes t is the temperature in degrees C * 10
    if(ModeF)
        deg = ((9*deg)/5)+320;

    bcd_data.digit0 = deg %10;
    deg /= 10;
    bcd_data.digit1 = deg % 10;
    deg /= 10;
    bcd_data.digit2 = deg % 10;
    deg /= 10;
    bcd_data.digit3 = deg % 10;    
}

void acquire_display_data(void)
{
    // acquire and convert selected display data
    // conversions are made at loop rate so data is
    // ready to be written in the short LCD WA window.
    
    switch(display_mode)
    {
        case MODE_TIME:
            acquire_time();
            break;
        case MODE_TEMPERATURE:
            // flag is set every 10 ms by TimeBaseManager
            if(TMR0_time_tic)
            {
                TMR0_time_tic = 0;
                // acquire new temperature every .5 sec
                if(--TMR0_temperature_timer == 0)
                {
               		acquire_temperature(FALSE);
          			   TMR0_temperature_timer = 50;
                }
           	}
            break;
        default:
            break;
    }
} 