// UNIO_Support.c
//
// Support files for UNIO memory access.
//
// The routines in this file are necessarily separate from Main.c because the
// typedefs in the UNIO driver files conflict with those in GenericTypeDefs.h
//
#include <xc.h>
#include "Main.h"
#include "UNIO PIC16.h"
#include "UNIO_support.h"
#include "EEData.h"

extern mode_motor motor_type;

extern unsigned char   add_on_revision;

WORD address;                // UNIO Address value

// wrappers to access UNIO init and restore routines and avoid typedef conflicts
void InitUNIO(void){UNIO_init();}
void RestoreUNIO(void){UNIO_restore();}

/************************************************************************
*                                                                       *
*      Function:       ReadAddOnType(void)                              *
*                                                                       *
*      Description:    Reads the Add-On board UNIO memory to determine  *
*                      the add-on type and revision.                    *
*                                                                       *
*      Parameters:                                                      *
*      Return value:   Global variables affected:                       *
*                      motor_type                                       *
*                      add_on_revision                                  *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

void ReadAddOnType(void)
{
	STATUSREG UNIO_status;
	
	// adjust Timer1 and CCP1 for UNIO control
	UNIO_init();
	
	// read UNIO stats to determine if memory is preset
	// status value of 0xFF indicates that no memory exists
	UNIO_status = ReadStatusReg();
	
	if (UNIO_status._byte == 0xFF)
	{
		motor_type = NONE;
	}
	else
	{
		// read last 4 locations for type and revision
		// extended address accomodates any size memory because address wraps for smaller memories
		address.byte0 = (unsigned char)(nvBoardIDBlock & 0xFF);
		address.byte1 = (unsigned char)((nvBoardIDBlock>>8) & 0xFF);
		SeqRead(address,4);
		add_on_revision = pageBuffer[1];
		
		// set motor type
		switch (pageBuffer[3])
		{
			case 2: // BLDC
				motor_type = BLDC;
				break;
		 	case 5: // LV Test board
				motor_type = TEST;
				break;
			case 6: // BDC
				motor_type = BDC;
				break;
			case 7: // Bipolar Stepper
				motor_type = BIPOLAR;
				break;
			case 8: // Unipolar Stepper
				motor_type = UNIPOLAR;
				break;
			default: // undefined or no board detected
				motor_type = UNDEFINED;
				break;
		}
	}
	// restore Timer1 and CCP1
	UNIO_restore();
}

/************************************************************************
*                                                                       *
*      Function:       WriteAddOnType(MT,Rev)                           *
*                                                                       *
*      Description:    Writes the Add-On board ID and revision to       *
*                      UNIO memory.                                     *
*                      Memory is written as follows:                    *
*                        Last location= MotorType ID                    *
*                        Last -1      = 0x00                            *
*                        Last -2      = Revision                        *
*                        Last -3      = 0x00                            *
*                                                                       *
*      Parameters:                                                      *
*      Return value:   Global variables affected:                       *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

void WriteAddOnType(unsigned char MT, unsigned char Rev)
{
	char* pb;
	STATUSREG UNIO_status;
	
	// adjust Timer1 and CCP1 for UNIO control
	UNIO_init();
	
	// determine if memory is code protected and unprotect if it is
	UNIO_status = ReadStatusReg();
	if(UNIO_status._byte != 0x00)
	{
		UNIO_status._byte = 0x00;
		WriteStatusReg(UNIO_status);
	} 
	
	// extended address accomodates any size memory because address wraps for smaller memories
	address.byte0 = (nvBoardIDBlock & 0xFF);
	address.byte1 = ((nvBoardIDBlock>>8) & 0xFF);
	
	pb = pageBuffer;
	*pb++ = 0x00;
	*pb++ = Rev;
	*pb++ = 0x00;
	*pb = MT;
	
	PageWrite(address,4);
	
	// code protect upper 1/4 of memory
	UNIO_status.BP0 = 1;
	WriteStatusReg(UNIO_status);
	
	// restore Timer1 and CCP1
	UNIO_restore();
}

/************************************************************************
*                                                                       *
*      Function:       ReadUNIOByte(address)                            *
*                                                                       *
*      Description:    Reads one byte of data from the UNIO memory      *
*                      at the specified address.                        *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                     Memory byte residing at address                   *
*                                                                       *
*      Note:                                                            *
*        UNIO_init be called before using this routine                  *
*        and UNIO_restore must be called when all reads are complete    *
*                                                                       *
*************************************************************************/

unsigned char ReadUNIOByte(unsigned int addr)
{
	address.byte0 = (unsigned char)(addr & 0x00FF);
	address.byte1 = (unsigned char)(addr >> 8);
	
	ByteRead(address);
	
	return pageBuffer[0];
}

/************************************************************************
*                                                                       *
*      Function:       WriteUNIOByte(addr,data)                         *
*                                                                       *
*      Description:    Writes one byte of data to the UNIO memory       *
*                      at the specified address.                        *
*                                                                       *
*      Parameters:                                                      *
*      Return value:   None                                             *
*                                                                       *
*      Note:                                                            *
*        UNIO_init be called before using this routine                  *
*        and UNIO_restore must be called when all writes are complete   *
*                                                                       *
*************************************************************************/

void WriteUNIOByte(unsigned int addr, unsigned char data)
{
	address.byte0 = (unsigned char)(addr & 0x00FF);
	address.byte1 = (unsigned char)(addr >> 8);
	
	pageBuffer[0] = data;
	ByteWrite(address);
	
}

/************************************************************************
*                                                                       *
*      Function:       WriteUNIOBlock(*buffer, addr, size)              *
*                                                                       *
*      Description:    Writes the buffer[size] of data to the UNIO      *
*                      memory starting at the specified address.        *
*                                                                       *
*      Parameters:                                                      *
*                   *buffer - pointer to the data to be written         *
*                    addr   - starting address of the write             *
*                    size   - buffer size                               *
*      Return value:                                                    *
*                   integer sum of all bytes written                    *
*                                                                       *
*      Note:                                                            *
*                   addr must be a page boundry. The validity of addr   *
*                   is not checked.                                     *
*************************************************************************/
unsigned int WriteUNIOBlock(char *buffer, unsigned int addr, int size)
{
    unsigned char *src;
    unsigned char *dest;
    int byte_count;
    unsigned int checksum;
    unsigned char page_bytes;
    
	// adjust Timer1 and CCP1 for UNIO control
	UNIO_init();
	
	address.byte0 = (unsigned char)(addr & 0x00FF);
	address.byte1 = (unsigned char)(addr >> 8);

    src = buffer;
    byte_count = size;
    checksum = 0;

    while(byte_count > 0)
    {
        for(page_bytes = PAGESIZE,
            dest = pageBuffer; 
            (page_bytes>0) && (byte_count>0); 
            --page_bytes,
            --byte_count)
        {
            checksum += *src;
            *dest++ = *src++;
        }
        page_bytes = PAGESIZE-page_bytes;
        PageWrite(address,page_bytes);
        address._word += PAGESIZE;
    }
        
	// restore Timer1 and CCP1
	UNIO_restore();
	return checksum;
}

/************************************************************************
*                                                                       *
*      Function:       ReadUNIOBlock(*buffer, addr, size)               *
*                                                                       *
*      Description:    Reads the buffer[size] of data from the UNIO     *
*                      memory starting at the target addr.              *
*                                                                       *
*      Parameters:                                                      *
*                   *buffer - pointer to target buffer                  *
*                    addr   - starting address of the read              *
*                    size   - bytes to read                             *
*      Return value:                                                    *
*                   negative integer difference of all bytes read       *
*                                                                       *
*      Note:                                                            *
*************************************************************************/
unsigned int ReadUNIOBlock(char *buffer, unsigned int addr, int size)
{
    unsigned char *src;
    unsigned char *dest;
    int byte_count;
    unsigned int checksum;
    unsigned char page_bytes;
    
	// adjust Timer1 and CCP1 for UNIO control
	UNIO_init();
	
	address.byte0 = (unsigned char)(addr & 0x00FF);
	address.byte1 = (unsigned char)(addr >> 8);

    dest = buffer;
    byte_count = size;
    checksum = 0;

    while(byte_count > 0)
    {
        SeqRead(address,PAGESIZE);
        for(page_bytes = PAGESIZE,
            src = pageBuffer; 
            page_bytes>0; 
            --page_bytes)
        {
            checksum -= *src;
            *dest++ = *src++;
            if(--byte_count == 0) break;
        }
        address._word += PAGESIZE;
    }
        
	// restore Timer1 and CCP1
	UNIO_restore();
	
	return checksum;
}

/************************************************************************
*                                                                       *
*      Function:       WriteUNIOWord(addr, data16)                      *
*                                                                       *
*      Description:    Writes the 16-bit word to the UNIO               *
*                      memory at the specified address.                 *
*                                                                       *
*      Parameters:                                                      *
*                    addr   - target address of the write               *
*                    data16 - 16-bit word                               *
*      Return value: none                                               *
*                                                                       *
*      Note:                                                            *
*            data is written lsByte in lowest address                   *
*        UNIO_init must be called before using this routine             *
*        and UNIO_restore must be called when all writes are complete   *
*                                                                       *
*************************************************************************/
void WriteUNIOWord(unsigned int addr, unsigned int data16)
{
    WriteUNIOByte(addr,(unsigned char)(data16&0xFF));
    WriteUNIOByte(++addr,(unsigned char)(data16>>8));
}

/************************************************************************
*                                                                       *
*      Function:       ReadUNIOWord(addr)                               *
*                                                                       *
*      Description:    Returns the 16-bit word from the UNIO            *
*                      memory at the specified address.                 *
*                                                                       *
*      Parameters:                                                      *
*                    addr   - target address of the read                *
*      Return value:                                                    *
*                    word at addr, lsByte is lowest address             *
*                                                                       *
*      Note:                                                            *
*        UNIO_init must be called before using this routine             *
*        and UNIO_restore must be called when all writes are complete   *
*                                                                       *
*************************************************************************/
unsigned int ReadUNIOWord(unsigned int addr)
{
    unsigned int data16;

    data16 = ReadUNIOByte(addr);
    data16 |= (ReadUNIOByte(++addr)<<8);
        
    return data16;
}

/************************************************************************
*                                                                       *
*      Function:       unsigned int WriteUNIOLong(addr, data32)         *
*                                                                       *
*      Description:    Writes the 32-bit word to the UNIO               *
*                      memory at the specified address.                 *
*                                                                       *
*      Parameters:                                                      *
*                    addr   - target address of the write               *
*                    data32 - 32-bit long                               *
*      Return value:                                                    *
*                    checksum - sum of all bytes written                *
*                                                                       *
*      Note:                                                            *
*            data is written lsByte in lowest address                   *
*        UNIO_init must be called before using this routine             *
*        and UNIO_restore must be called when all writes are complete   *
*                                                                       *
*************************************************************************/
unsigned int WriteUNIOLong(unsigned int addr, unsigned long data32)
{
    unsigned char tempc;
    unsigned int checksum = 0;
    unsigned char ShiftCount;
    
    for(ShiftCount = 0; ShiftCount<4; ShiftCount++)
    {
	    tempc = (data32 & 0xFF);
	    checksum += tempc;
	    WriteUNIOByte(addr++, tempc);
	    data32 >>= 8;
	}
	
	return checksum;
}

/************************************************************************
*                                                                       *
*      Function:       unsigned int ReadUNIOLong(addr, *data32)         *
*                                                                       *
*      Description:    Reads the 32-bit word from the UNIO              *
*                      memory at the specified address.                 *
*                                                                       *
*      Parameters:                                                      *
*                    addr   - target address of the read                *
*                    data32 - pointer to 32-bit long                    *
*      Return value:                                                    *
*                    checksum - sum of all bytes read                   *
*                                                                       *
*      Note:                                                            *
*            data is retrieved lsByte in lowest address                 *
*        UNIO_init must be called before using this routine             *
*        and UNIO_restore must be called when all writes are complete   *
*                                                                       *
*************************************************************************/
unsigned int ReadUNIOLong(unsigned int addr, unsigned long *data32)
{
    unsigned char tempc;
    unsigned int checksum = 0;
    unsigned char ShiftCount;
    
    addr += 3;
    *data32 = 0;
    for(ShiftCount = 0; ShiftCount<4; ShiftCount++)
    {
	    tempc = ReadUNIOByte(addr--);
	    checksum += tempc;
	    *data32 |= tempc;
	    if(ShiftCount<3) *data32 <<= 8;
	}
	
	return checksum;
}

/************************************************************************
*                                                                       *
*      Function:       WriteUNIOStatus(unsigned char x)                 *
*                                                                       *
*      Description:    Write x to the UNIO status register              *
*                                                                       *
*      Parameters:                                                      *
*                    x   - target byte for the status register          *
*      Return value:                                                    *
*                    none                                               *
*                                                                       *
*      Note:                                                            *
*        UNIO_init be called before using this routine                  *
*        and UNIO_restore must be called when all other UNIO access     *
*        is complete.                                                   *
*                                                                       *
*************************************************************************/
void WriteUNIOStatus(unsigned char x)
{
	STATUSREG srTemp;
	
	srTemp._byte = x;
	WriteStatusReg(srTemp);
}	

/************************************************************************
*                                                                       *
*      Function:       ClearUNIOMemory(void)                            *
*                                                                       *
*      Description:    Write 0xFF to all UNIO memory                    *
*                                                                       *
*      Parameters:                                                      *
*                    None                                               *
*      Return value:                                                    *
*                    none                                               *
*                                                                       *
*      Note:                                                            *
*        WriteUNIOStatus must be called to clear the write protect bits *
*        before using this routine and UNIO_restore must be called      *
*        when all other UNIO access is complete.                        *
*                                                                       *
*************************************************************************/
void ClearUNIOMemory(void)
{
    SetAll();
}	
