// lcd2.c

/*********************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************/
#include <xc.h>
#include "lcd2.h"
#include "seven_seg.h"

const UINT8  segment_data[] = { 
                                 DIGIT_0, 
                                 DIGIT_1, 
                                 DIGIT_2, 
                                 DIGIT_3, 
                                 DIGIT_4, 
                                 DIGIT_5, 
                                 DIGIT_6, 
                                 DIGIT_7, 
                                 DIGIT_8, 
                                 DIGIT_9, 
                                 DIGIT_A, 
                                 DIGIT_B, 
                                 DIGIT_C, 
                                 DIGIT_D, 
                                 DIGIT_E, 
                                 DIGIT_F};

/****************************************************************************
  Function:
    void lcd_init(void)

  Summary:
    Initializes the LCD peripheral for the F1 Evaluation Platform

  Description:
    This function configures the LCD peripheral for:
    Type B Waveform,
    Timer 1 Oscillator, and 
    1:1 prescaler
    All segment data is cleared.  The segments enables are configured for
    the F1 Evaluation Platform hardware.
 
  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    Type B waverorm was used to allow the LCDIF flag to be the RTCC source.
    This feature was needed when operating the BLDC motors because Timer 1
    is needed for accurate commutation time and cannot be used for RTCC.
  ***************************************************************************/
void lcd_init(void)
{

    // Configure LCDPS
    // Wave form type A or B
    // Bias Mode
    // Prescaler
    // 1:16 - 1:1
    LCDPS = 0;
    WFT = 1;        // B type
    LCDPSbits.LP = 0;

    T1OSCEN = 1;    // activate the 32khz oscillator for our clock source

    /*******************************************************/

    /* User is responsible to enable the needed segments   */

    /* Replace the code in the next 3 lines for your glass */

    /*******************************************************/
    LCDSE0 = 0x8B;
    LCDSE1 = 0x0F;
    LCDSE2 = 0x0C;
    LCDSE3 = 0x60;
    LCDSE4 = 0x06;
    LCDSE5 = 0x18;

    /*******************************************************/

    // Configure LCDCON
    // LCD enbled
    // LCD enabled in sleep
    // LCD Clock Select - T1OSC
    // LCD Commons - 1/4
    LCDCON = 0;
    SLPEN = 0;
    WERR = 0;
    LCDCONbits.CS = 1;
    LCDCONbits.LMUX = 3;

    // clear ALL SEGMENT DATA
    LCDDATA0 = 0;
    LCDDATA1 = 0;
    LCDDATA2 = 0;
    LCDDATA3 = 0;
    LCDDATA4 = 0;
    LCDDATA5 = 0;
    LCDDATA6 = 0;
    LCDDATA7 = 0;
    LCDDATA8 = 0;
    LCDDATA9 = 0;
    LCDDATA10 = 0;
    LCDDATA11 = 0;
    LCDDATA12 = 0;
    LCDDATA13 = 0;
    LCDDATA14 = 0;
    LCDDATA15 = 0;
    LCDDATA16 = 0;
    LCDDATA17 = 0;
    LCDDATA18 = 0;
    LCDDATA19 = 0;
    LCDDATA20 = 0;
    LCDDATA21 = 0;
    LCDDATA22 = 0;
    LCDDATA23 = 0;

    LCDIF = 0;

    // Reference ladder control
    LCDRL = 0x30;

    // Configure LCDREF
    // Internal reference enabled
    // Internal Reference Source
    // Internal Reference always on
    LCDREF = 0;
    LCDIRE = 1;
    LCDIRS = 0;
    LCDIRI = 0;

    LCDCST = 0;     // maximum contrast
    LCDEN = 1;
}

/****************************************************************************
  Function:
    void lcd_display_off(void)

  Summary:
    Turn off the LCD peripheral.  Leave the display configured.

  Description:
    This function is needed to generate a blinking display without modifying
    all the segment data.
 
  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    
  ***************************************************************************/
void lcd_display_off(void)
{
    LCDEN = 0;
}

/****************************************************************************
  Function:
    void lcd_display_on(void)

  Summary:
    Turn on the LCD peripheral.  Leave the display configured.

  Description:
    This function is needed to generate a blinking display without modifying
    all the segment data.
 
  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    
***************************************************************************/
void lcd_display_on(void)
{
    LCDEN = 1;
}

/****************************************************************************
  Function:
    void lcd_display_digits(BCD_TYPE b)

  Summary:
    Render the 4 BCD digits 0-9,A-F passed in BCD_TYPE

  Description:
    This function accepts the numbers for display on the LCD. It renders the 
    7 segment data as quickly as possible by a 2 level mapping of digits to 
    segments and then segments to LCD segments.
 
  Precondition:
    WA should be true (Type B waveform only)

  Parameters:
    BCD_TYPE b : digits to render

  Returns:
    None

  Remarks:
    The following applies to Type B waveform mode only:
    The WA (Write Allow) bit must be tested true before executing this routine.
    Since WA is true for a limited time at a relatively slow duty cycle
    the calling routine should execute this routine immediately
    after the WA rising edge.
  ***************************************************************************/

void lcd_display_digits(BCD_TYPE b)
{
    lcd_display_character(4,segment_data[b.digit0]);
    lcd_display_character(3,segment_data[b.digit1]);
    lcd_display_character(2,segment_data[b.digit2]);
    lcd_display_character(1,segment_data[b.digit3]);
}

void lcd_display_digits_2(BCD_TYPE_2 b)
{
    lcd_display_character(6,segment_data[b.digit0]);
    lcd_display_character(5,segment_data[b.digit1]);
}
/****************************************************************************
  Function:
    void lcd_display_LS_digits(BCD_TYPE b)

  Summary:
    Render the 4 BCD digits 0-9,A-F passed in BCD_TYPE
    Leading zeros are blanked
    
  Description:
    This function accepts the numbers for display on the LCD. It renders the 
    7 segment data as quickly as possible by a 2 level mapping of digits to 
    segments and then segments to LCD segments.
 
  Precondition:
    WA should be true (Type B waveform only)

  Parameters:
    BCD_TYPE      b : digits to render
    INT8     digits : number of unblanked LS digits. LS digit is always unblanked
    
  Returns:
    None

  Remarks:
    When digits == 2 then the two LS digits are unblanked, etc.
    The following applies to Type B waveform mode only:
    The WA (Write Allow) bit must be tested true before executing this routine.
    Since WA is true for a limited time at a relatively slow duty cycle
    the calling routine should execute this routine immediately
    after the WA rising edge.
  ***************************************************************************/

void lcd_display_LS_digits(BCD_TYPE b,INT8 digits)
{
    
    // blank MS digits from top down
    if(b.digit3 == 0 && digits < 4)
    {
        lcd_display_character(1,BLANK);
        if(b.digit2 == 0 && digits < 3)
        {
            lcd_display_character(2,BLANK);
            if(b.digit1 == 0 && digits < 2)
            {
                lcd_display_character(3,BLANK);
            }
            else
            {
                lcd_display_character(3,segment_data[b.digit1]);
            }
        }
        else
        {
            lcd_display_character(2,segment_data[b.digit2]);
            lcd_display_character(3,segment_data[b.digit1]);
        }
    }
    else
    {
        lcd_display_character(1,segment_data[b.digit3]);
        lcd_display_character(2,segment_data[b.digit2]);
        lcd_display_character(3,segment_data[b.digit1]);
    }

    // always display the LS digit
    lcd_display_character(4,segment_data[b.digit0]);
    
}

/****************************************************************************
  Function:
    void lcd_display_character(char DigitNumber, UINT8 Character)

  Summary:
    Render the character passed in on the specified DigitNumber. Each bit in the character
    represents the on/off state of the corresponding 7-segment display where bit 0 is segment A
    through bit 6 which is segment G. Bit 7 is not used.

  Description:
    This function accepts the numbers for display on the LCD. It renders the 
    7 segment data as quickly as possible by a 2 level mapping of digits to 
    segments and then segments to LCD segments.
 
  Precondition:
    WA should be true (Type B waveform only)

  Parameters:
    UINT8 DigitNumber : Glass digit number. Digit number 1 is left-most on glass of 6 digits.
    UINT8 Character   : 7-segment display bits

  Returns:
    None 

  Remarks:
    The following applies to Type B waveform mode only:
    The WA (Write Allos) bit must be tested true before executing this routine.
    Since WA is true for a limited time at a relatively slow duty cycle
    the calling routine should execute this routine immediately
    after the WA rising edge.
  ***************************************************************************/

void lcd_display_character(UINT8 DigitNumber, UINT8 Character)
{
// The long list of IF statements was the fastest code as of HITEC 9.65PL1
    
    switch (DigitNumber)
    {
        case 1:
            if(Character & SEG_A)
            {
                A1 = 1;
            }
    
            if(!(Character & SEG_A))
            {
                A1 = 0;
            }
    
            if(Character & SEG_B)
            {
                B1 = 1;
            }
    
            if(!(Character & SEG_B))
            {
                B1 = 0;
            }
    
            if(Character & SEG_C)
            {
                C1 = 1;
            }
    
            if(!(Character & SEG_C))
            {
                C1 = 0;
            }
    
            if(Character & SEG_D)
            {
                D1 = 1;
            }
    
            if(!(Character & SEG_D))
            {
                D1 = 0;
            }
    
            if(Character & SEG_E)
            {
                E1 = 1;
            }
    
            if(!(Character & SEG_E))
            {
                E1 = 0;
            }
    
            if(Character & SEG_F)
            {
                F1 = 1;
            }
    
            if(!(Character & SEG_F))
            {
                F1 = 0;
            }
    
            if(Character & SEG_G)
            {
                G1 = 1;
            }
    
            if(!(Character & SEG_G))
            {
                G1 = 0;
            }
            break;
        case 2:
            if(Character & SEG_A)
            {
                A2 = 1;
            }
    
            if(!(Character & SEG_A))
            {
                A2 = 0;
            }
    
            if(Character & SEG_B)
            {
                B2 = 1;
            }
    
            if(!(Character & SEG_B))
            {
                B2 = 0;
            }
    
            if(Character & SEG_C)
            {
                C2 = 1;
            }
    
            if(!(Character & SEG_C))
            {
                C2 = 0;
            }
    
            if(Character & SEG_D)
            {
                D2 = 1;
            }
    
            if(!(Character & SEG_D))
            {
                D2 = 0;
            }
    
            if(Character & SEG_E)
            {
                E2 = 1;
            }
    
            if(!(Character & SEG_E))
            {
                E2 = 0;
            }
    
            if(Character & SEG_F)
            {
                F2 = 1;
            }
    
            if(!(Character & SEG_F))
            {
                F2 = 0;
            }
    
            if(Character & SEG_G)
            {
                G2 = 1;
            }
    
            if(!(Character & SEG_G))
            {
                G2 = 0;
            }            
            break;
        case 3:
            if(Character & SEG_A)
            {
                A3 = 1;
            }
    
            if(!(Character & SEG_A))
            {
                A3 = 0;
            }
    
            if(Character & SEG_B)
            {
                B3 = 1;
            }
    
            if(!(Character & SEG_B))
            {
                B3 = 0;
            }
    
            if(Character & SEG_C)
            {
                C3 = 1;
            }
    
            if(!(Character & SEG_C))
            {
                C3 = 0;
            }
    
            if(Character & SEG_D)
            {
                D3 = 1;
            }
    
            if(!(Character & SEG_D))
            {
                D3 = 0;
            }
    
            if(Character & SEG_E)
            {
                E3 = 1;
            }
    
            if(!(Character & SEG_E))
            {
                E3 = 0;
            }
    
            if(Character & SEG_F)
            {
                F3 = 1;
            }
    
            if(!(Character & SEG_F))
            {
                F3 = 0;
            }
    
            if(Character & SEG_G)
            {
                G3 = 1;
            }
    
            if(!(Character & SEG_G))
            {
                G3 = 0;
            }
            break;
        case 4:
            if(Character & SEG_A)
            {
                A4 = 1;
            }
    
            if(!(Character & SEG_A))
            {
                A4 = 0;
            }
    
            if(Character & SEG_B)
            {
                B4 = 1;
            }
    
            if(!(Character & SEG_B))
            {
                B4 = 0;
            }
    
            if(Character & SEG_C)
            {
                C4 = 1;
            }
    
            if(!(Character & SEG_C))
            {
                C4 = 0;
            }
    
            if(Character & SEG_D)
            {
                D4 = 1;
            }
    
            if(!(Character & SEG_D))
            {
                D4 = 0;
            }
    
            if(Character & SEG_E)
            {
                E4 = 1;
            }
    
            if(!(Character & SEG_E))
            {
                E4 = 0;
            }
    
            if(Character & SEG_F)
            {
                F4 = 1;
            }
    
            if(!(Character & SEG_F))
            {
                F4 = 0;
            }
    
            if(Character & SEG_G)
            {
                G4 = 1;
            }
    
            if(!(Character & SEG_G))
            {
                G4 = 0;
            }            
            break;
        case 5:
            if(Character & SEG_A)
            {
                A5 = 1;
            }
    
            if(!(Character & SEG_A))
            {
                A5 = 0;
            }
    
            if(Character & SEG_B)
            {
                B5 = 1;
            }
    
            if(!(Character & SEG_B))
            {
                B5 = 0;
            }
    
            if(Character & SEG_C)
            {
                C5 = 1;
            }
    
            if(!(Character & SEG_C))
            {
                C5 = 0;
            }
    
            if(Character & SEG_D)
            {
                D5 = 1;
            }
    
            if(!(Character & SEG_D))
            {
                D5 = 0;
            }
    
            if(Character & SEG_E)
            {
                E5 = 1;
            }
    
            if(!(Character & SEG_E))
            {
                E5 = 0;
            }
    
            if(Character & SEG_F)
            {
                F5 = 1;
            }
    
            if(!(Character & SEG_F))
            {
                F5 = 0;
            }
    
            if(Character & SEG_G)
            {
                G5 = 1;
            }
    
            if(!(Character & SEG_G))
            {
                G5 = 0;
            }            
            break;
        case 6:
            if(Character & SEG_A)
            {
                A6 = 1;
            }
    
            if(!(Character & SEG_A))
            {
                A6 = 0;
            }
    
            if(Character & SEG_B)
            {
                B6 = 1;
            }
    
            if(!(Character & SEG_B))
            {
                B6 = 0;
            }
    
            if(Character & SEG_C)
            {
                C6 = 1;
            }
    
            if(!(Character & SEG_C))
            {
                C6 = 0;
            }
    
            if(Character & SEG_D)
            {
                D6 = 1;
            }
    
            if(!(Character & SEG_D))
            {
                D6 = 0;
            }
    
            if(Character & SEG_E)
            {
                E6 = 1;
            }
    
            if(!(Character & SEG_E))
            {
                E6 = 0;
            }
    
            if(Character & SEG_F)
            {
                F6 = 1;
            }
    
            if(!(Character & SEG_F))
            {
                F6 = 0;
            }
    
            if(Character & SEG_G)
            {
                G6 = 1;
            }
    
            if(!(Character & SEG_G))
            {
                G6 = 0;
            }            
            break;
        default:
            // ignore bogus requests
            break;
    }// switch
}