/*///////////////////////////////////////////////////////////////////////////
//
// File: Monitor.c
//
// C based serial port monitor program.
//
/////////////////////////////////////////////////////////////////////////////
*                
*
*********************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************
*
* Change History:
* Author               Date        Comment

* w.r.brown            2011.08.23  V 3-1 -  Added boolean AND, OR, and XOR on target address
* w.r.brown            2011.08.22  V 2a  -  Added "L"inear memory access command
* w.r.brown            2010.07.14  V 2   -  Started keeping track of history
*******************************************************************************************************/
#define MONITOR_VERSION ".3-1"

#include <xc.h>
#include "Main.h"
#include "Monitor.h"
#include "UartInterface.h"
#include "MonitorTests.h"

extern bit MSG_READY;
extern bit ECHO_ON;
extern bit TXON;

near unsigned char FSRHSave;
near unsigned char FSRLSave;
near unsigned char RxData;

far char* rpString;
extern const char rsMainVersion[1];
extern const char rsTestVersion[1];
extern const char rsSpeedVersion[1];
extern const char rsControlVersion[1];
extern const char rsStartupType[1];

const char rsMonitorVersion[] = MONITOR_VERSION;
const char rsVersion[] = "F1 LV Demo ";
const char rsEscape[] = " Escape ";
const char rsMenu[] = 
	"\rAxxx     : View addr xxx "
	"\rAxxx=yy  : Change addr xxx to yy "
	"\rAxxx&yy  : Addr xxx ANDed with yy"
	"\rAxxx|yy  : Addr xxx ORed with yy "
	"\rAxxx^yy  : Addr xxx XORed with yy"
	"\rLxxxx    : View linear region addr xxxx"
	"\rLxxxx=yy : Change linear region addr xxxx to yy"
	"\r         : &, |, and ^ operators also accepted "
//	"\rBxxx,y   : View bit y at addr xxx"
//	"\rBxxx,y=z : Change bit y at addr to z "
	"\rV        : Show version"
	"\rTnn      : invoke user test number nn"
	"\r         :  where 00<=nn<=FF (0 to 255)"
	"\rE        : Toggle echo off/on"
	"\r<esc>    : Abort command "
	"\r<bkspc>  : Remove last character "	
	"\r?        : Show menu";

enum {TX_SERVICE, TX_STRING} TxKey;
enum {DECODE_FIRST_CHAR,
	   SEND_VERSION,
	   SHOW_MENU,
	   ADDR_0,
	   ADDR_1,
	   ADDR_2,
	   ADDR_3,
	   ADDR_4,
	   ADDR_5,
	   ADDR_6,
	   ADDR_7,
	   TEST_1,
	   TEST_2,
	   TEST_3,
      TOGGLE_ECHO,
	   SEARCH_FOR_CR
 } CmdKey;
 
enum {EQUAL, OR, AND, XOR} DataKey;

extern union FLAGS Flags; 
union SPLIT_BYTE WorkByte;

/////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////// SUBROUTINES //////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

void MonitorInit(void)
{
// Full initialization for the monitor
   
   // Setup internal oscillator frequency
   OSCCON = OSCCON_INIT;
   OPTION_REG = OPTION_REG_INIT;
   	
	USARTInit();
	AppInit();
//	NewLine();
}
//------------------------------------------------------------------------------------
void AppInit(void)
{
	TXON = 0;
	MSG_READY = 0;
	ECHO_ON = 1;  // default to echo on
	CmdKey = DECODE_FIRST_CHAR;
	TxKey = TX_SERVICE;
}
//------------------------------------------------------------------------------------
void ServiceCommunicator(void)
{
	TxmtService();
	RcvService();
	CmdService();
}
//------------------------------------------------------------------------------------
void NewLine(void)
{
	// Send a carriage return to the TX buffer
	
	PutTx(0x0D);
	SendPrompt();
}
//------------------------------------------------------------------------------------
void CmdService(void)
{
	if (!MSG_READY) return;
	RxData = GetRx();
	// Command service routine goes here
	switch (CmdKey)
	{
		case DECODE_FIRST_CHAR:
			DecodeFirstChar();
			break;
		case SEND_VERSION:
			if (RxData == '\r')
			{ 
	   			// combined total number of characters in all versions must be
	   			// less than the TxBuffer size.
	   			FillTxBuffer((char*)rsVersion);
	   			FillTxBuffer((char*)rsMainVersion);
	   			FillTxBuffer((char*)rsMonitorVersion);
	   			FillTxBuffer((char*)rsTestVersion);
	   			//FillTxBuffer((char*)rsSpeedVersion);
	   			//FillTxBuffer((char*)rsControlVersion);
	   			//FillTxBuffer((char*)rsStartupType);
	   			NewLine();
   			}
			else CmdKey = SEARCH_FOR_CR;
			break;
		case SHOW_MENU:
			if (RxData == '\r') StartString((char*)rsMenu);
			else CmdKey = SEARCH_FOR_CR;
			break;
		case ADDR_0:
			DecodeFSRHm();
			break;
		case ADDR_1:
			DecodeFSRH();
			break;
		case ADDR_2:
			DecodeMSNyb();
			break;
		case ADDR_3:
			DecodeLSNyb();
			FSRLSave = WorkByte.val;
			break;
		case ADDR_4:
			CmdKey++;	
		    switch (RxData)
		    {
		        case '=':
		            DataKey = EQUAL;
		            break;
		        case '|':
		            DataKey = OR;
		            break;
		        case '&':
		            DataKey = AND;
		            break;
		        case '^':
		            DataKey = XOR;
		            break;
		        case '\r':
		            SendAddrData();
		            break;
		        default:
    				CmdKey = SEARCH_FOR_CR;
    				break;
    	    }	                 
			break;
		case ADDR_5:
			DecodeMSNyb();
			break;
		case ADDR_6:
			DecodeLSNyb();
			break;
		case ADDR_7:
			if (RxData == '\r') SetAddrData();
			else  CmdKey = SEARCH_FOR_CR;
			break;
		case TEST_1:
			DecodeMSNyb();
			break;
		case TEST_2:
			DecodeLSNyb();
			break;
		case TEST_3:
			if (RxData == '\r')
			{
				// force the expected prompt before executing the Test
				// to prevent PC from timing out waiting for the response
				PutTx('>');
				TxService();
				Test(WorkByte.val);
				CmdKey = DECODE_FIRST_CHAR;
			}
			else
				CmdKey = SEARCH_FOR_CR;
			break;
		case TOGGLE_ECHO:
			if (RxData == '\r') 
			{
				ECHO_ON = ~ECHO_ON;
				SendPrompt();
			}
			else  CmdKey = SEARCH_FOR_CR;
			break;
		case SEARCH_FOR_CR:
			// ignore everything until a CR is received
			// when a CR is recognized then reset the command
			// state machine and generate a new line with prompt
			if (RxData == '\r') SendPrompt();
			break;
		default:
			CmdKey = SEARCH_FOR_CR;
			break;
	}
}
//------------------------------------------------------------------------------------
void RcvService(void)
{
	if(!RCIF) return;
	
	// retrieve one character from the USART fifo into the RxBuffer
	switch (RxService())
	{
		case BREAK:
			MonitorInit();
			break;
		case ESCAPE:
			ResetRxPointers();
			MSG_READY = 0;
			StartString((char*)rsEscape);
			break;
		case END_OF_TEST:
			// end of serial link integrity test detected
			// reset the state machines and input buffer
			ResetRxPointers();
			TxKey = TX_SERVICE;
			CmdKey = DECODE_FIRST_CHAR;
			break;
		default:
			// No action on normal characters which were put in RxBuffer by RxService();
			// when the end of line character is received then the MSG_READY flag is set
			// which starts the command processor.
			break;
	}
}
//------------------------------------------------------------------------------------
void TxmtService(void)
{
	//	Send one character in the transmit buffer to the USART
	//	In some cases one character is also loaded into the transmit buffer.
	
	if (TXON == 0) return;
	if (TXIF == 0) return;
	
	switch (TxKey)
	{
		case TX_SERVICE:
			TxService();
			break;
		case TX_STRING:
			SendString();
			break;
		default:
			break;
	}
}
//------------------------------------------------------------------------------------
void StartString(far char *sp)
{
	 TxKey = TX_STRING;
	 rpString = sp;
	 SendString();
}
//------------------------------------------------------------------------------------
void SendString(void)
{
	// Send one character of the pString buffer to the EUSART
	// All strings are followed by the new line and prompt character.
	
	if (*rpString != 0x00)
	{
		PutTx(*rpString++);
		TxService();
	}
	else
	{
		TxKey = TX_SERVICE;
		NewLine();
	}
}
//------------------------------------------------------------------------------------
void SendPrompt(void)
{
	// A prompt is the final act of all monitor functions.
	// The prompt character indicates to the host that the monitor is finished
	// processing the previous command and is ready for the next command.
	// Reset the Command and Transmit state machines then
	// send the prompt character
	
	CmdKey = DECODE_FIRST_CHAR;
//	TxKey = TX_SERVICE;
	PutTx('>');
}
//------------------------------------------------------------------------------------
void DecodeFirstChar(void)
{
	//
	
	switch (RxData)
	{
		case '\r':	// CR terminate
			SendPrompt();
			break;
		case 'V':	// show version
			CmdKey = SEND_VERSION;
			break;
		case '?':   // Menu
			CmdKey = SHOW_MENU;
			break;
		case 'A':	// address command
		    FSRHSave = 0;
			CmdKey = ADDR_1;
			break;
		case 'E':	// Toggle echo
			CmdKey = TOGGLE_ECHO;
			break;
		case 'T':	// Test mode
//			TestNumber = 0;
			CmdKey = TEST_1;
			break;
//		case 'B':	// bit command
        case 'L':   // linear address command
            CmdKey = ADDR_0;
            break;
		default:
			CmdKey = SEARCH_FOR_CR;
			break;
	}
}
//------------------------------------------------------------------------------------
union SPLIT_BYTE AsciiToHex(unsigned char a)
{
	// convert one ascii hex character to a 4-bit result
	union SPLIT_BYTE result;
	
	if ((a >= 'a') && (a <= 'f'))
		result.val = (a-'a'+ 0xA);
	else if ((a >= '0') && (a <= '9')) 
		result.val =  (a-'0');
	else if ((a >='A') && (a <= 'F')) 
		result.val =  (a - 'A' + 0xA);
	else
		result.bits.b4 = 1;

	return result;
}
//------------------------------------------------------------------------------------
unsigned char HexToAscii(struct NYBBLE n)
{
	// convert a 4 bit nybble to one ascii hex character
	
	if (n.val < 0x0A) return (n.val + '0');
	return (n.val + ('A'-0x0A));
}	
//------------------------------------------------------------------------------------
void DecodeFSRHm(void)
{
	union SPLIT_BYTE sb;
	if (RxData == '\r')
	{
		SendPrompt();
		return;
	}
	sb = AsciiToHex(RxData);
	if (sb.bits.b4 == 0)
	{
		FSRHSave = sb.nybbles.ls<<4;
		CmdKey++;
	}
	else
	{
		CmdKey = SEARCH_FOR_CR;
	}
}
//------------------------------------------------------------------------------------
void DecodeFSRH(void)
{
	union SPLIT_BYTE sb;
	if (RxData == '\r')
	{
		SendPrompt();
		return;
	}
	sb = AsciiToHex(RxData);
	if (sb.bits.b4 == 0)
	{
		FSRHSave |= sb.nybbles.ls;
		CmdKey++;
	}
	else
	{
		CmdKey = SEARCH_FOR_CR;
	}
}
//------------------------------------------------------------------------------------
void DecodeLSNyb(void)
{
	union SPLIT_BYTE sb;
	if (RxData == '\r')
	{
		SendPrompt();
		return;
	}
	sb = AsciiToHex(RxData);
	if (sb.bits.b4 == 0)
	{
		WorkByte.nybbles.ls = sb.nybbles.ls;
		CmdKey++;
	}
	else
	{
		CmdKey = SEARCH_FOR_CR;
	}
}
//------------------------------------------------------------------------------------
void DecodeMSNyb(void)
{
	DecodeLSNyb();
	WorkByte.nybbles.ms = WorkByte.nybbles.ls;
}
//------------------------------------------------------------------------------------
void SendAddrData(void)
{
	struct NYBBLE temp;
	
	FSR0H = FSRHSave;
	FSR0L = WorkByte.val;
	WorkByte.val = INDF0;
	temp.val = WorkByte.nybbles.ms;
	PutTx(HexToAscii(temp));
	temp.val = WorkByte.nybbles.ls;
	PutTx(HexToAscii(temp));
	NewLine();
}	
//------------------------------------------------------------------------------------
void SetAddrData(void)
{
	
	FSR0H = FSRHSave;
	FSR0L = FSRLSave;
	switch(DataKey)
	{
    	case AND:
	        INDF0 &= WorkByte.val;
	        break;
    	case OR:
	        INDF0 |= WorkByte.val;
	        break;
    	case XOR:
	        INDF0 ^= WorkByte.val;
	        break;
    	default:
    	//case EQUAL:
	        INDF0 = WorkByte.val;
	        break;
	}
	SendPrompt();
}	
//------------------------------------------------------------------------------------
void FillTxBuffer(far char *sp)
{
   // send all contents of string in *sp to TxBuffer
   while (*sp != 0x00) PutTx(*sp++);;
}