/*///////////////////////////////////////////////////////////////////////////
//
// File: UartInterface.c
//
// USART wrappers for C based monitor program.
//
/////////////////////////////////////////////////////////////////////////////
*                
*
*********************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************
*
* Change History:
* Author               Date        Comment

* w.r.brown            2010.07.28  V 3   -  Added RxCount and inhibit TX when RxBuffer full.
* w.r.brown            2010.07.28  V 3   -  Added RxCount and inhibit TX when RxBuffer full.
* w.r.brown            2010.07.14  V 2   -  Started keeping track of changes.
*                                           Added button selectable AUTOBAUD.
*******************************************************************************************************/
//#include <stdio.h>
//#include <usart.h>
//#include <timers.h>
#include <xc.h>
#include "Main.h"
#include "Monitor.h"
#include "UartInterface.h"
#include "EEData.h"

//near union FLAGS Flags;
//near enum IMMEDIATE RxImmediate;
near unsigned char TxCount;
near unsigned char RxCount;

unsigned char RxBuffer[32];
unsigned char TxBuffer[64];

unsigned char *pRxPut;
unsigned char *pTxPut;
unsigned char *pRxGet;
unsigned char *pTxGet;
unsigned char *pRxBuffEnd;
unsigned char *pTxBuffEnd;

bit MSG_READY;
bit ECHO_ON;
bit TXON;

volatile char cTemp;

/////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////// SUBROUTINES //////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

void ResetRxTx(void)
{
	// Reset the buffer pointers and flags

	TXON = 0;
	MSG_READY = 0;
	pRxPut = pRxGet = RxBuffer;
	pRxBuffEnd = RxBuffer + sizeof(RxBuffer);
	pTxPut = pTxGet = TxBuffer;
	pTxBuffEnd = TxBuffer + sizeof(TxBuffer);
	TxCount = 0;
	RxCount = 0;
}
//------------------------------------------------------------------------------------
void USARTInit(void)
{	
	ResetRxTx();
    NV_SPBRG_Read();
    
    // Force default 38400 baud if EE value is erased
    if (SPBRG == 0xFF)
    {
        SPBRG = 0x34;
       	NV_SPBRG_Save();
    }   

    // When the communications interface opens the com port it issues a
    // break which causes USART initialization. Ohly perform an Autobaud
    // if the PushButton is pressed during this sequence. This prevents
    // unnecessary wear on the baudrate EEPROM storage byte.
    if (PushButton == 0)
    {
       	Autobaud();
       	NV_SPBRG_Save();
    }
    else
    {	
       	SPEN = 1;
       	CREN = 1;
       	BRGH = 1;
    }
    TXEN = 1;
}
//------------------------------------------------------------------------------------
void Autobaud(void)
{   
   SPEN = 0;
   SenseBreak();
   BRGH = 1;
   SPEN = 1;
   CREN = 1;
// hardware autobaud detect: Requires "U" calibration character
   ABDEN = 1;
   while(RCIF == 0) CLRWDT();
   while(RCIF == 1) cTemp = RCREG;
   
   // SPBRG is set at end of this routine.
}   
//------------------------------------------------------------------------------------
void WaitForRise(void)
{
// wait for rising edge on RX line
	while(RX == 1);	// wait for low
	while(RX == 0);	// exit on rise
	return;
}
//------------------------------------------------------------------------------------
void SenseBreak(void)
{
// Wait until a break is detected on the RX pin.
// A break will stay low for at least one Timer0 overflow.
// At Fosc=32MHz, Tmr0 prescale = 256:1, and Tmr0 preloaded
// with 0x05, an overflow will occur after about 8 ms.
   unsigned char temp;
   
#define UART_TIMER0_PRESCALE     256L
#define TMR0_BREAK_MS            8L
#define TMR0_BREAK_COUNT         (TMR0_BREAK_MS*SYSTEM_FREQUENCY)/(UART_TIMER0_PRESCALE*MILLISECONDS_PER_SEC)
#define TMR0_BREAK_PRESET        (0x00FF-TMR0_BREAK_COUNT)
	
	// save current Timer0 settings
	temp = OPTION_REG;   
	// Change Timer0 prescaler to extend maximum count period
	OPTION_REG = 0b10000111;
	
	do
	{
		while(RX == 1) CLRWDT();	// wait for low
		TMR0 = TMR0_BREAK_PRESET;
		T0IF = 0;
		while(RX == 0)    // continue on rise
	   {  CLRWDT();
         LATA4 = !LATA4; // Diagnostic
      }

	}while (T0IF == 0);
	// restore OPTION_REG;
	OPTION_REG = temp;
}
//------------------------------------------------------------------------------------
void PutcUsart(char c)
{
	while(!TXIF);
	TXREG = c;
// append line feeds to carriage returns
	if (c == 0x0D) PutcUsart(0x0A);
}
//------------------------------------------------------------------------------------
void PutTx(unsigned char t)
{
	// Move the character t into the circular TxBuffer
	// and request TX service
	
	*pTxPut++ = t;
	if (pTxPut == pTxBuffEnd) pTxPut = TxBuffer;
	TxCount++;
	// inhibit echos when receive buffer gets full
	if(RxCount < sizeof(RxBuffer))
	   TXON = 1;
	else
	   TXON = 0;
}
//------------------------------------------------------------------------------------
void TxService(void)
{
	//	Move a character from the TX buffer space to the TX output
	//	register. Disable the serial transmitter if the TX buffer is empty.
	
	if (pTxGet != pTxPut)
	{
		if(!TXIF) return;
		TXREG = *pTxGet++;
		if(pTxGet == pTxBuffEnd) pTxGet = TxBuffer;
		TxCount--;
		return;
	}
	TXON = 0;
}
//------------------------------------------------------------------------------------
enum IMMEDIATE RxService(void)
{
/****************************************************************************
	Move a character from the RX input register to the RX buffer space.
	All characters are placed in the input buffer space except 
	backspace, which deletes the last character. A carrige return 
	terminates the receive message and sets the MsgFlag to indicate
 	that a new message is in the receive buffer.
	This routine echos all received characters, except BREAK, if the echo flag
	is true.
	The IMMEDIATE return variable is so that some special characters can be
	processed immediately by the calling routine	instead of waiting in thereceive
	buffer for the command processor.
*****************************************************************************/
	unsigned char RxChar;
	
	if(!RCIF) return NORMAL;
	// clear Overrun error when detected
	if(OERR)
	{
   	CREN = 0;
   	CREN = 1;
   	while(RCIF)RxChar = RCREG;
   	return NORMAL;
   }
	if(FERR == 1)
	{
      RxChar = RCREG;
   	if(RxChar == 0x00) return BREAK;
   }
   else
   {
      RxChar = RCREG;
   }     	

	// echo received character by sending it to the output buffer
	if (ECHO_ON == 1) PutTx(RxChar);
	
	if (RxChar == '\b') // backspace
	{
		if (pRxPut != pRxGet)
		{
   		RxCount--;
			if (pRxPut == RxBuffer)
			{
   			pRxPut = pRxBuffEnd - 1;
         }			 
			else
			{
   			pRxPut--;
         }			
		}	
	}
	else
	{
		*pRxPut++ = RxChar;
		if (pRxPut == pRxBuffEnd)
		{
   		pRxPut = RxBuffer;
      }
      RxCount++; 		
	}
	// immediate processing
	switch (RxChar)
	{
		case 0x1B:
			return ESCAPE;
		case 0x7E: 
			return END_OF_TEST;
//		case '\b':  // backspace
//			return NORMAL;
		case '\r':
			MSG_READY = 1;
			break;
		default:
			break;
	}
	return NORMAL;
}
//------------------------------------------------------------------------------------
void ResetRxPointers(void)
{
	pRxPut = pRxGet;
	RxCount = 0;
}
//------------------------------------------------------------------------------------
unsigned char GetRx(void)
{
	// return the next character from the receive input buffer
	unsigned char c;
	
	c = *pRxGet++;
	if (pRxGet == pRxBuffEnd) pRxGet = RxBuffer;
	if (pRxGet == pRxPut)
	{
   	MSG_READY = 0;
   	RxCount = 0;
   }
   else
	   RxCount--;
	
	return c;
}
//------------------------------------------------------------------------------------
char TxReady(void)
{
	// returns true (0x01) if the TxBuffer is not full
	
	if (TxCount < (sizeof(TxBuffer)-3)) return (0x01);  // buffer is not full - return true
	return (0x00);
}
