/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : W.R.Brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : BLDC_Interrupts_Combo.c                    *
*     Date                 : 2009/03/12                                 *
*     SharePoint Version   : 1.1                                        *
*                                                                       *
*     Tools Used: MPLAB GL : 8.14                                       *
*                 Compiler : Hi-Tech                                    *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************
*
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//   Combined closed loop and open loop operation                                                     //
//   Closed loop interrupts have two states: Zero_detect and Commutate.                               //
//   This version also includes dynamic blanking.                                                     //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
*
*
*********************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************
*
* Change History:
* Author               Date        Comment
* w.r.brown            2010.09.30  V 1.3  - Changed advance and balance offsets to degrees instead of time
* w.r.brown            2010.07.28  V 1.2  - Commented voltage sensing out
*                                           Added constant for control type identification
*                                           Corrected advance and added timing balance adjustments
* w.r.brown            2009.03.22  V 1.1  - Added voltage sensing to stall detection
* w.r.brown            2009.03.15  V 1.0  - First release
*******************************************************************************************************/
#include <xc.h>
#include "Main.h"

/************************************************************************
* variable definitions                                                  *
*************************************************************************/
const char rsControlVersion[] = "CCP 1.3 ";

extern doublebyte TMR1_comm_time;
extern unsigned char TMR0_stall_timer;

extern bit startup_complete_flag;
extern bit stop_flag;
extern bit rising_bemf_flag;
extern bit measure_bemf_flag;

enum {high_res_setup,zero_detect,commutate}isr_state;

extern int zc_error;
extern int temp;
extern doublebyte expected_zc;
extern doublebyte zc;
extern char cTemp;
extern doublebyte comm_after_zc;
extern char tach_timer;
extern char comm_state;
doublebyte CCP_comm;

int CommOffset;
int timing_balance;
int Offset15;
int Offset7p5;
int Offset3p75;
char CommOffsetRequest;
char TimingBalanceRequest;
char TempRequest;

extern unsigned char blanking_count;
extern unsigned char error_scale;

// variables for open loop

extern volatile doublebyte InputData;
extern bit SpeedReady;
extern bit OffsetReady;
extern bit BalanceReady;
bit OpenLoopFlag;
bit SingleStep;

////////////////////////////////////////////////////////////////////////////////////////////////////////
//    Brushless motor drive is controlled by interrupts. There are two types of interrupts:           //
//    ECCP4 Compare and BEMF comparator.                                                              //
//                                                                                                    //
//    ECCP4 Compare :                                                                                 //
//       Commutation interrupt - Motor drive is changed to the next commutation phase.                //
//       This is also the start of the blanking time when the upcomming period is rising BEMF (for    //
//       high-side modulation) or falling BEMF (for low-side modulation).                             //
//       CCPR4 is set to match Timer1 at the commutation time. If the BEMF_FLAG is set then           //
//       blanking is performed and then the the comparator interrupt is enabled after clearing the    //
//       comparator interrupt-on-change registers otherwise commutation correction calculations       //
//       are performed basd on the previous zero-crossing event data.                                 //
//    Comparator interrupt:                                                                           //
//       Zero Cross interrupt - This signals when the BEMF voltage has crossed the motor supply       //
//       voltage midpoint. Timer1 is read and the measured time is compared to the expected time.     //
//       The error time difference is used to adjust both the upcomming commutation and the predicted //
//       commutation period for future commutations. Commutation computations are performed in the    //
//       next commutation state when there is more time because blanking and zero cross are not       //
//       performed.                                                                                   //
////////////////////////////////////////////////////////////////////////////////////////////////////////
      
void brushless_interrupt_handler(void) {
   
   if(OpenLoopFlag)
   {
      // run open loop when TMR1_comm_time result is greater than arbitrary minimum
      // otherwise single step on every press of SW2
      if ((unsigned int)(TMR1_comm_time.bytes.high) < 0xFF)
      {
         Commutate();
      }      
      else
      {
         // SingleStep flag can also be set by PC
         if(SingleStep)
         {
            Commutate();
            SingleStep = 0;
         }
      }
      // prevent update when InputData is in transition
      // PC sets SpeedReady flag to indicate that the InputData is stable
      // 
      if (SpeedReady)
      {
         if(InputData.word == 0)
            InputData.word = 1;
         TMR1_comm_time.word = -InputData.word;
         // reload match registers with latest speed setting
         CCPR4H = TMR1_comm_time.bytes.high;
         CCPR4L = TMR1_comm_time.bytes.low;
         SpeedReady = 0;
      }
      TP1 = 0; // Diagnostic
      CCP4IF = 0;   
   }
   else
   {      
      switch (isr_state)
      {
         case zero_detect:
            // disable comparator interrupts
            CxIE = 0;
            // this is either an ECCP4 interrupt or a comparator interrupt
            // if it's a comparator interrupt then zero cross has been detected and needs to be processed
            // if it's a ECCP4 interrupt then the zero cross was missed so we need to skip zero cross
            // processing and fall through to commutation.
            if(CxIF)
            {
               TP2=1;                          //    Diagnostic - Zero detection
               do{
                  zc.bytes.high = TMR1H;
                  zc.bytes.low = TMR1L;
               }
               while (zc.bytes.high != TMR1H);
               
               if(startup_complete_flag)
               {
                  CCP_comm.word = zc.word + comm_after_zc.word;
                  CCPR4H = CCP_comm.bytes.high;
                  CCPR4L = CCP_comm.bytes.low;
               }
               /*
               // stall detection
               // The measure_bemf_flag was set by StallControl()which is waiting
               // for the flag to be cleared
               if(measure_bemf_flag)
               {
                  // coordinate BEMF measurement with the peak of a known driven phase
                  // StallControl() has already setup the ADC to watch this phase
                 if(comm_state==BEMF_STATE)
                 {
                    // disable the driven phase
                    BEMF_DRIVE_BIT = 0;
                    // wait for flyback currents to settle
                    cTemp = TMR1L + BEMF_DELAY_COUNT;
                    while (TMR1L != cTemp);
                    GO_nDONE = 1;
                    // wait for ADC acquisition time
                    cTemp = TMR1L + ACQUISITION_DELAY_COUNT;
                    while (TMR1L != cTemp);
                    measure_bemf_flag = 0;
                    // restore the driven phase
                    BEMF_DRIVE_BIT = 1;
                 }
               }
               */     
               TP2 = 0;  // Diagnostic
               CCP4IF = 0;
               isr_state = commutate;
               break;
            }
            // if execution reaches this point then no zero cross was detected
            // this only happens during deceleration and when searching for zero cross in forced commutation
            // if we're decelerating then adjust the commutation time to catch up with the motor
            
             if(startup_complete_flag)
             {
                // shorten by 1/8 commutation cycle
                TMR1_comm_time.word -= (expected_zc.word>>2);
             }
            
         case commutate:
            // Commutation occurs when Timer3 matches CCPR4
            if(CCP4IF)
            {
               //CCP4IF = 0;
               //    This service commutates the motor and either waits for a blanking period or computes the next commutation.
               //    If BEMF_FLAG indicates that zero cross detection is to be performed in the the upcomming period then blanking is performed.
               //    Blanking holds off the input to the comparator to allow the commutation switching transients to settle.
               //    A relatively clean BEMF signal will then be available for zero cross detection.
               //    In this version of the software blanking is performed dynamically. We wait a minimum blanking period to
               //    give the drivers a chance to settle then we read the comparator output until it is low thereby ensuring
               //    that the flyback currents have settled. Then, the comparator will be setup for zero cross detection.
               Commutate();
               if(BEMF_FLAG)
               {
                  // dynamic blanking
                  // wait a mimimum blanking time to allow drivers to settle
                  CCP_comm.word = blanking_count;
                  CCPR4H = CCP_comm.bytes.high;
                  CCPR4L = CCP_comm.bytes.low;
                  CCP4IF = 0;
                  while(!CCP4IF);
                  // setup CCP4 for the full commutation period
                  CCPR4H = TMR1_comm_time.bytes.high;
                  CCPR4L = TMR1_comm_time.bytes.low;
                  CCP4IF = 0;
                  // wait for flyback currents to settle before setting up comparator for interrupts
                  // if ECCP4 interrupts while waiting then flyback voltage and zero cross were both
                  //   missed in which case we need to commutate and try again.
                  if (startup_complete_flag)
                     while(CxOUT) if (CCP4IF) break;  
                  cTemp = CMxCON0;   // reading control register clears mis-match flops
                  CxIF = 0;
                  CxIE = 1;
                  isr_state = zero_detect;
                  TP1 = 0; // Diagnostic
               }   
               else
               {
                  // Compute corrected commutation time from present commutation time and zero-crossing event time.
                  // Zero-crossing error is the difference between when ZC should have happened and when it actually happened.
                  // Negative error shortens the commutation time and positive error lengthens the commutation time.
                  //
                  // ZCE(n) = ZC(n)-(CT(n)/2)
                  // CT(n+1) = CT(n) + ZCE(n)*Error_Gain
                  // where:
                  //   CT is commutation time
                  //   ZC is zero-crossing event time
                  //   ZCE is zero-crossing error
                  //   (n) denotes present cycle
                  //   (n+1) denotes next cycle
                  //
                  // Timer1 is a 16-bit timer. When using the compare register to measure time then Timer1 is always
                  // positive, even when the MSb is 1. Think of Timer1 as a 17-bit number where the MSb is always 0.
                  // The MSb is always masked to zero to keep the result positive to prevent the MSb from extending
                  // (sign extension) when right shifting to compute half the time.
                  
                  //expected_zc.word = (TMR1_comm_time.word>>1) & 0x7FFF; // expected_zc is positive time to zero cross
                                                                        // strip sign extension
                  
                  // timing is advanced/retarded by subtracting an offset from the ZC event and then extending
                  // the same amount of time time to commutation after the ZC occurs. 
                  zc.word += CommOffset;
                  zc.word -= timing_balance;
                  
                  zc_error = zc.word - expected_zc.word;           // negative: too early, positive: too late
                  temp = zc_error;
                  if(temp & 0x8000) temp = ~temp+1;  // absolute value
                  // stop forced commutation if zero cross detected within middle half of comm period
                  if (temp < Offset15)
                  {
                    startup_complete_flag = 1;
                    TMR0_stall_timer = TIMEBASE_STALL_COUNT;
                    TP0 = 0; // Diagnostic
                  }
                  else
                  {
                    TP0 = 1;  // Diagnostic
                  }
                  
                  // Systems ramp up at the maximum rate determined by the difference
                  // between the zero cross event (which usually happens immediately after blanking)
                  // and the middle of the expected commutation time.
   
                  TMR1_comm_time.word += (zc_error>>error_scale);       // accumulate error

                  // setup for commutation
                  CCPR4H = TMR1_comm_time.bytes.high;
                  CCPR4L = TMR1_comm_time.bytes.low;
                  //if((unsigned int)TMR1_comm_time.word > MAX_TMR1_PRESET) stop_flag=1;
   
                  isr_state = commutate;
                  
                  expected_zc.word = (TMR1_comm_time.word>>1) & 0x7FFF; // expected_zc is positive time to zero cross
                                                                        // strip sign extension
                  Offset15 = expected_zc.word>>1;
                  Offset7p5 = Offset15>>1;
                  Offset3p75 = Offset7p5>>1;
                  
                  // Build offsets relative to commutation time from last recorded half-commutation time
                  CommOffset = 0;
                  TempRequest = CommOffsetRequest;
                  if((TempRequest & 0x80) != 0) TempRequest = -TempRequest;
                  if((0x01 & TempRequest)!=0) CommOffset = Offset3p75;
                  if((0x02 & TempRequest)!=0) CommOffset += Offset7p5;
                  if((0x04 & TempRequest)!=0) CommOffset += Offset15;
                  if((CommOffsetRequest & 0x80)!= 0) CommOffset = -CommOffset;
                  
                  timing_balance = 0;
                  TempRequest = TimingBalanceRequest;
                  if((TempRequest & 0x80) != 0) TempRequest = -TempRequest;
                  if((0x01 & TempRequest)!=0) timing_balance = Offset3p75;
                  if((0x02 & TempRequest)!=0) timing_balance += Offset7p5;
                  if((0x04 & TempRequest)!=0) timing_balance += Offset15;
                  if((TimingBalanceRequest & 0x80)!= 0) timing_balance = -timing_balance;
                                                                        
                  // comm_after_zc is the forced time remaining to commutation after the ZC event occurs.
                  // the advance/retard time, subtracted from the measured ZC event, is added here to 
                  // compensate for the time difference. This makes the closed loop control respond as
                  // it would have if the ZC event was actually advanced or retarded.
                  // timing_balance adjusts for differences in zero-cross and commutation-only periods

                  comm_after_zc.word = expected_zc.word + CommOffset + timing_balance;
                  
                  // synchronize CommOffset and timing_balance changes to prevent changes while the variable is being used
                  // OffsetReady and BalanceReady are set by the PC control after the InputData buffer is ready
                  if(OffsetReady)
                  {
                     CommOffsetRequest = InputData.bytes.low;
                     OffsetReady = 0;
                  }
                  if(BalanceReady)
                  {
                     TimingBalanceRequest = InputData.bytes.low;
                     BalanceReady = 0;
                  }
               }
               CCP4IF = 0;            
            }
            break;
         default:
            stop_flag = 1;
            break;
      }/* end switch */
   }

}/* end ISR */

