/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : w.r.brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : BrakeControl.c                             *
*     Date                 : 2010/10/07                                 *
*                                                                       *
*     Other Files Required :                                            *
*     Tools Used: MPLAB GL : 8.14                                       *
*                 Compiler : Hi-Tech                                    *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************
*
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//    BLDC braking and brake recovery.                                                                //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
*/

#include <xc.h>
#include "Main.h"
#include "BrakeControl.h"
#include "Init.h"

void MeasureBEMF(void);

extern bit TMR0_brake_flag;
extern bit BrakeRequest;
extern bit run_arm_flag;
extern bit run_flag;

extern unsigned char BrakeLevel;
enum {BrakeIdle, BrakeActive, BrakeRecover} BrakeState;

extern bit stop_flag;
extern doublebyte TMR1_comm_time;
extern doublebyte zc;
extern doublebyte expected_zc;
extern doublebyte comm_after_zc;
extern int Offset15;
extern unsigned char ramped_speed;
extern enum {high_res_setup,zero_detect,commutate}isr_state;

/************************************************************************
*                                                                       *
*      Function:       BrakeControl                                     *
*                                                                       *
*      Description:    Suspend interrupts and apply braking percent     *
*                      to all three modulated drivers.                  *
*                      Brake recovery reads all three BEMF levels to    *
*                      determine recovery drive level then applies that *
*                      level before enabling commutation interrupts.    *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

void BrakeControl(void) 
{   
   unsigned int PhaseAVoltage;
   unsigned int PhaseBVoltage;
   unsigned int PhaseCVoltage;
   unsigned int RefVoltage;
   unsigned int MaxPhaseVoltage;
   char CommState;
   
   if(!TMR0_brake_flag) return;
   
   TMR0_brake_flag = 0;
   
   switch(BrakeState)
   {
      case BrakeIdle:
         if(!BrakeRequest)
            break;
      // BrakeStart:
         GIE = 0;        // this stops commutation
         run_flag = 0;   // this prevents PWM changes by speed control
         GetCCPVal(BrakeLevel);
         TurnBrakeOn();
         BrakeState = BrakeActive;
         break;
      case BrakeActive:
         // stay in braking condition until BrakeRequest is released
         if(BrakeRequest)
            break;
      case BrakeRecover:
         // remove braking modulation
         TurnBrakeOff();
         // measure BEMF reference voltage
         ADFM = 1;  // right justify result
         ADCON0 = ADCON0_BEMF_REF;
         MeasureBEMF();
         RefVoltage = ADRES;
         // measure voltage on all three phases
         ADCON0 = ADCON0_PHASE_A;
         MeasureBEMF();
         PhaseAVoltage = ADRES;
         ADCON0 = ADCON0_PHASE_B;
         MeasureBEMF();
         PhaseBVoltage = ADRES;
         ADCON0 = ADCON0_PHASE_C;
         MeasureBEMF();
         PhaseCVoltage = ADRES;
         ADFM = 0;  // restore left justified result
         // determine maximum BEMF and comm state
         if(PhaseAVoltage > PhaseBVoltage)
            if(PhaseCVoltage > PhaseAVoltage)   // must be comm state 1, 2, or 6
            {
               MaxPhaseVoltage = PhaseCVoltage;
               CommState = 6;
            }
            else
            {
               MaxPhaseVoltage = PhaseAVoltage; // must be comm state 1 or 2
               if(PhaseBVoltage > PhaseCVoltage)
                  CommState = 2;
               else
                  CommState = 1;
            }
         else                                   // must be comm state 3, 4, or 5
            if(PhaseCVoltage > PhaseBVoltage)
            {   
               MaxPhaseVoltage = PhaseCVoltage;
               CommState = 5;
            }
            else
            {
               MaxPhaseVoltage = PhaseBVoltage;  // must be comm state 3 or 4
               if(PhaseCVoltage > PhaseAVoltage)
                  CommState = 4;
               else
                  CommState = 3;
            }
         PhaseBVoltage = MaxPhaseVoltage<<7;
         // determine ratio of motor voltage to reference
         PhaseAVoltage = PhaseBVoltage/RefVoltage;
         // set drive level comparable to BEMF
         ramped_speed = (unsigned char)PhaseAVoltage;
         GetCCPVal(ramped_speed);        
         // Determine DAC level equivalent to 1/2 MaxPhaseVoltage
         // MaxPhaseVoltage is 10-bit number of which upper 5-bits equate to desired DAC setting
         // Shift MaxPhaseVoltage right 5+1 bits for DAC setting
         PhaseBVoltage = MaxPhaseVoltage>>6;
         if(PhaseBVoltage == 0)
         {
            // motor too slow to detect
            // start from stop condition
            stop_flag = 1;
            break;
         }
#ifdef HIGH_SIDE_MODULATION
         // set comparator channel to capture next low to high transition
         // commutation state is set to state following the transition
         switch (CommState)
         {
            case 4:
            case 5:
            case default:
               COMPARATOR = SENSE_A;
               comm_state = 1;
               break;
            case 1:
            case 6:
               COMPARATOR = SENSE_B;
               comm_state = 3;
               break;
            case 2:
            case 3:
               COMPARATOR = SENSE_C;
               comm_state = 5;
               break;
          }              
#else
         // set comparator channel to capture next high to low transition
         // commutation state is set to state foll0wing the transition
         switch (CommState)
         {
            case 1:
            case 2:
            default:
               COMPARATOR = SENSE_A;
               CommState = 4;
               break;
            case 3:
            case 4:
               COMPARATOR = SENSE_B;
               CommState = 6;
               break;
            case 5:
            case 6:
               COMPARATOR = SENSE_C;
               CommState = 2;
               break;
          }              
#endif
         DACEN = 1; // enable DAC
         DACCON1 = (char)PhaseBVoltage;
         // setup C1 to use DAC as the reference
         C1PCH0 = 1;
         // disable Timer1 and clear the count
         TMR1ON = 0;
         TMR1H = 0;
         TMR1L = 0;
         // setup Timer1 gate to capture commutation time
         // Gate enabled, toggle mode, low true gate, C1 source
         T1GCON = 0b10110010;
         TMR1ON = 1;
         // start gate capture
         T1GGO = 1;
         // wait for gate done to indicate successful capture
         // watchdog will prevent getting stuck here
         while(T1GGO);
         // restore C1 to use pin as the reference, turn off DAC, and turn off Timer1 Gate
         C1PCH0 = 0;
         DACEN = 0;
         T1GCON = 0;
         // Time in Timer1 is commutation time
         // initialize all commutation variables
         TMR1_comm_time.bytes.high = TMR1H;
         TMR1_comm_time.bytes.low = TMR1L;
         
         expected_zc.word = (TMR1_comm_time.word>>1) & 0x7FFF;
         zc.word = expected_zc.word;
         comm_after_zc.word = expected_zc.word;
         Offset15 = expected_zc.word>>1;
         // preset commutation compare for full comm period
         // preset Timer1 to simulate the half period that has elapsed.
         isr_state = commutate;
         TMR1ON = 0;
         TMR1H = expected_zc.bytes.high;
         TMR1L = expected_zc.bytes.low;
         CCPR4H = TMR1_comm_time.bytes.high;
         CCPR4L = TMR1_comm_time.bytes.low;
         TMR1ON = 1;
         CxIE = 0;
         CCP4IF = 0;
         CCP4IE = 1;
         run_arm_flag = 1;  // allows speed manager to set run_flag
         // re-enable interrupts and hope to reaquire lock
         BrakeState = BrakeIdle;
         GIE = 1;
         break;
      default:
         BrakeState = BrakeIdle;
         break;
   }  
}

void MeasureBEMF(void)
{
   // during braking recovery Timer1 and CCPR4 are not used for commutation
   // so we're sharing that resource to time BEMF acquisition.
   // wait acquisition time
   TMR1ON = 0;
   TMR1H = 0;
   TMR1L = 0;
   CCPR4H = (0xFF & (BRAKE_DELAY_COUNT/256L));
   CCPR4L = (0xFF & BRAKE_DELAY_COUNT);
   CCP4IF = 0;
   TMR1ON = 1;
   while(!CCP4IF);
   // start ADC
   GO_nDONE = 1;
   while(GO_nDONE);
}

void TurnBrakeOn(void)
{
   // disable all fixed drives and enable all PWM drives
   DRIVE_U_BIT    = 0;               // Fixed drive off
   DRIVE_V_BIT    = 0;
   DRIVE_W_BIT    = 0;
   MODULATE_U_BIT = 1;
   MODULATE_V_BIT = 1;
   MODULATE_W_BIT = 1;
}

void TurnBrakeOff(void)
{
   // disable all PWM drives
   MODULATE_U_BIT = 0;
   MODULATE_V_BIT = 0;
   MODULATE_W_BIT = 0;
}

void BrakeInit(void)
{
   BrakeState = BrakeIdle;
   BrakeRequest = 0;
}   