/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : W.R.Brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : DirectBitDrivers_Combo.c                   *
*     Date                 : 2010/08/12                                 *
*     SharePoint Version   : 1.1                                        *
*                                                                       *
*     Tools Used: MPLAB GL : 8.01                                       *
*                 Compiler : Hi-Tech                                    *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
************************************************************************/

////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
// Bit control of discrete FET driver commutation states.                                             //
// Combo mode combines: Open loop and closed loop, both with reversible direction                     //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////


/*******************************************************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*******************************************************************************************************
*
*
*******************************************************************************************************
*
* Change History:
* Author               Date        Comment
* w.r.brown            2010.10.07  V 0.3  - Added braking
* w.r.brown            2010.08.18  V 0.2  - Added indicators
* w.r.brown            2010.08.12  V 0.1  - Derived from DirectDrivers_Combo.c
*******************************************************************************************************/

#include <xc.h>
#include "Main.h"
#include "BrakeControl.h"

extern unsigned char comm_state; 
extern bit rising_bemf_flag;
extern bit ReverseDirection;

/************************************************************************
*                                                                       *
*      Function:       Commutate                                        *
*                                                                       *
*      Description:    driver commutation                               *
*                                                                       *
*      PSTRCON is used to steer PWM to one side of bridge and           *
*      DRIVE_PORT is used to drive other side. Comparator input         *
*      is steered to undriven phase of motor.                           *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*************************************************************************/

//
//    4|5|6|1|2|3|4|5|6|1|2|3|
//     | | |_|_| | | | |_|_| |
//  U _|_|/| | |\|_|_|/| | |\|
//    _| | | | |_|_| | | | |_|
//  V  |\|_|_|/| | |\|_|_|/| |
//     |_|_| | | | |_|_| | | |
//  W /| | |\|_|_|/| | |\|_|_|
//
// State  Low   High  Comparator
//   1     V     U       -W
//   2     W     U        V
//   3     W     V       -U
//   4     U     V        W
//   5     U     W       -V
//   6     V     W        U
//
void Commutate(void)
{
#ifdef HIGH_SIDE_MODULATION
   // High side modulation
   // BEMF is sensed when rising
   switch(comm_state) {
      case 1:
         MODULATE_W_BIT    = 0;
         DRIVE_W_BIT       = 0;
         MODULATE_U_BIT    = 1;
         DRIVE_V_BIT       = 1;
         PHASE_W_INDICATOR = 0;
         PHASE_V_INDICATOR = 1;
         COMPARATOR        = SENSE_W_FALLING;
         rising_bemf_flag  = ReverseDirection;
         if(ReverseDirection)
            comm_state = 7;         // comm_state decrement will reset this to 6
      break;
      case 2:
         TP1=1;                    // Diagnostic
         MODULATE_V_BIT    = 0;
         DRIVE_V_BIT       = 0;
         MODULATE_U_BIT    = 1;
         DRIVE_W_BIT       = 1;
         PHASE_V_INDICATOR = 0;
         PHASE_W_INDICATOR = 1;
         COMPARATOR        = SENSE_V_RISING;
         rising_bemf_flag  = !ReverseDirection;
       //TP1=0;                  // Diagnostic
      break;
      case 3:
         MODULATE_U_BIT    = 0;
         DRIVE_U_BIT       = 0;
         MODULATE_V_BIT    = 1;
         DRIVE_W_BIT       = 1;
         PHASE_U_INDICATOR = 0;
         PHASE_W_INDICATOR = 1;
         COMPARATOR        = SENSE_U_FALLING;
         rising_bemf_flag  = ReverseDirection;
      break;
      case 4:
         MODULATE_W_BIT    = 0;
         DRIVE_W_BIT       = 0;
         MODULATE_V_BIT    = 1;
         DRIVE_U_BIT       = 1;
         PHASE_W_INDICATOR = 0;
         PHASE_U_INDICATOR = 1;
         COMPARATOR        = SENSE_W_RISING;
         rising_bemf_flag  = !ReverseDirection;
      break;
      case 5:
         MODULATE_V_BIT    = 0;
         DRIVE_V_BIT       = 0;
         MODULATE_W_BIT    = 1;
         DRIVE_U_BIT       = 1;
         PHASE_V_INDICATOR = 0;
         PHASE_U_INDICATOR = 1;
         COMPARATOR        = SENSE_V_FALLING;
         rising_bemf_flag  = ReverseDirection;
      break;
      case 6:
         MODULATE_U_BIT    = 0;
         DRIVE_U_BIT       = 0;
         MODULATE_W_BIT    = 1;
         DRIVE_V_BIT       = 1;
         PHASE_U_INDICATOR = 0;
         PHASE_V_INDICATOR = 1;
         COMPARATOR        = SENSE_U_RISING;
         rising_bemf_flag  = !ReverseDirection;
         if(!ReverseDirection)
            comm_state=0;                  // comm_state increment will reset this to 1
      break;
      default:
         PSTR1CON    = 0x00;            // Modulate off
         PSTR2CON    = 0x00;            // Modulate off
         PSTR3CON    = 0x00;            // Modulate off
         DRIVE_U_BIT = 0;               // Fixed drive off
         DRIVE_V_BIT = 0;
         DRIVE_W_BIT = 0;
         COMPARATOR = 0x00;
         rising_bemf_flag = 0;
         comm_state=2;
      break;
   };
#else
   // Low side modulation
   // BEMF is sensed when falling
   switch(comm_state) {
      case 1:
         MODULATE_W_BIT    = 0;
         DRIVE_W_BIT       = 0;
         MODULATE_V_BIT    = 1;
         DRIVE_U_BIT       = 1;
         PHASE_W_INDICATOR = 0;
         PHASE_U_INDICATOR = 1;
         COMPARATOR        = SENSE_W_FALLING;
         rising_bemf_flag  = ReverseDirection;
         if(ReverseDirection)
            comm_state = 7;         // comm_state decrement will reset this to 6
      break;
      case 2:
         MODULATE_V_BIT    = 0;
         DRIVE_V_BIT       = 0;
         MODULATE_W_BIT    = 1;
         DRIVE_U_BIT       = 1;
         PHASE_V_INDICATOR = 0;
         PHASE_U_INDICATOR = 1;
         COMPARATOR        = SENSE_V_RISING;
         rising_bemf_flag  = !ReverseDirection;
      break;
      case 3:
         MODULATE_U_BIT    = 0;
         DRIVE_U_BIT       = 0;
         MODULATE_W_BIT    = 1;
         DRIVE_V_BIT       = 1;
         PHASE_U_INDICATOR = 0;
         PHASE_V_INDICATOR = 1;
         COMPARATOR        = SENSE_U_FALLING;
         rising_bemf_flag  = ReverseDirection;
      break;
      case 4:
         MODULATE_W_BIT    = 0;
         DRIVE_W_BIT       = 0;
         MODULATE_U_BIT    = 1;
         DRIVE_V_BIT       = 1;
         PHASE_W_INDICATOR = 0;
         PHASE_V_INDICATOR = 1;
         COMPARATOR        = SENSE_W_RISING;
         rising_bemf_flag  = !ReverseDirection;
      break;
      case 5:
         TP1=1;                         // Diagnostic
         MODULATE_V_BIT    = 0;
         DRIVE_V_BIT       = 0;
         MODULATE_U_BIT    = 1;
         DRIVE_W_BIT       = 1;
         PHASE_V_INDICATOR = 0;
         PHASE_W_INDICATOR = 1;
         COMPARATOR        = SENSE_V_FALLING;
         rising_bemf_flag  = ReverseDirection;
       //TP1=0;                       // Diagnostic
      break;
      case 6:
         MODULATE_U_BIT    = 0;
         DRIVE_U_BIT       = 0;
         MODULATE_V_BIT    = 1;
         DRIVE_W_BIT       = 1;
         PHASE_U_INDICATOR = 0;
         PHASE_W_INDICATOR = 1;
         COMPARATOR        = SENSE_U_RISING;
         rising_bemf_flag  = !ReverseDirection;
         if(!ReverseDirection)
            comm_state=0;                  // comm_state increment will reset this to 1
      break;
      default:
         PSTR1CON    = 0x00;            // Modulate off
         PSTR2CON    = 0x00;            // Modulate off
         PSTR3CON    = 0x00;            // Modulate off
         DRIVE_U_BIT = 0;               // Fixed drive off
         DRIVE_V_BIT = 0;
         DRIVE_W_BIT = 0;
         COMPARATOR  = 0x00;
         rising_bemf_flag = ReverseDirection;
         comm_state=2;
      break;
   };
#endif   
   // setup for next commutation event
   if(ReverseDirection)   
      comm_state--;
   else
      comm_state++;
   
} // end Commutate
