/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : w.r.brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : Startup_CCP_OpenLoop.c                     *
*     Date                 : 2010/07/09                                 *
*                                                                       *
*     Other Files Required :                                            *
*     Tools Used: MPLAB GL : 8.14                                       *
*                 Compiler : Hi-Tech                                    *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************
*
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//    Routines to regulate operation during warmup and startup.                                       //
//    This version sets an initial commutation, voltage, and commutation period then enables          //
//    interrupts and lets the ISR do the rest. When the Open Loop Dwell period has elapsed then       //
//    the OpenLoop flag is reset and the motor continues operation in closed loop mode.               //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
********************************************************************************************************
* Change History:
* Author               Date        Comment
*
* w.r.brown            2012.04.13  V 0.4   - Cleared TMR1H (TMR1L was cleared twice) in SLOWSTEP
* w.r.brown            2011.07.21  V 0.3   - Restructured to use startup state machine instead of semiphores.
* w.r.brown            2010.10.08  V 0.2   - Changed open loop STARTINIT to set TMR1_comm_time to 0xFFFF
*                                            and call GetCCPVal(0) to set 0% PWM duty cycle . 
* w.r.brown            2010.08.13  V 0.1   - Started keeping track of changes. 
*                                            Accomodated zero time in open loop startup. 
********************************************************************************************************
*/


#include <xc.h>
#include "Main.h"
#include "Startup.h"
#include "Init.h"

// open loop startup type
const char rsStartupType[] = "OL 0.4";

extern enum {high_res_setup,zero_detect,commutate}isr_state;
enum {STARTINIT, SLOWSTEP, STEPDWELL, OPENLOOPRUN} SlowStartState;
extern doublebyte TMR1_comm_time;

extern char slow_start_events;
extern unsigned int TMR0_run_state_timer;
extern unsigned char TMR0_supply_timer;

extern bit startup_complete_flag;
extern bit stop_flag;
extern bit run_flag;

bit OpenLoopFlag;
extern unsigned char SlowStartDwell;
extern unsigned char OpenLoopDwell;
extern unsigned char ramped_speed;
extern volatile char startup_dutycycle;
extern volatile unsigned int startup_comm_time;
extern mode_run run_mode;

/************************************************************************
*                                                                       *
*      Function:       ControlSlowStart                                 *
*                                                                       *
*      Description:  Dwell at one or more successive commutations       *
*                    before starting the spin up                        *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*  This is called every main loop cycle but the TMR0_run_state_flag     *
*  limits execution to every 10 ms as defined by the TimeBaseManager()  *
*  routine.                                                             *
*                                                                       *
*************************************************************************/

void ControlSlowStart(void)
{   
   // The slow start timer determines how long to dwell at each slow
   // start commutation point.
   switch (SlowStartState)
   {
      case STARTINIT:
         // suppress minimum duty cycle when in PC open loop control mode
         if(OpenLoopFlag)
         {
            GetCCPVal(0);
            TMR1_comm_time.word = 0xFFFF;
            CCPR4H = TMR1_comm_time.bytes.high;
            CCPR4L = TMR1_comm_time.bytes.low;
            CCP4CON = CCP4CON_INIT;
            CCP4IF = 0;
            TMR1ON = 1;
            CCP4IE = 1;
            PEIE=1;   
            GIE=1;
            run_mode = RUN;
            break;
         }
         // startup duty cycle is set by SupplyManager()
         ramped_speed = FindTableIndex(startup_dutycycle);
         GetCCPVal(ramped_speed);
         SlowStartState = SLOWSTEP;
         
      case SLOWSTEP:
         if(slow_start_events > 0)
         {
            slow_start_events--;
            TMR0_run_state_timer = SlowStartDwell;
            SlowStartState = STEPDWELL;
            Commutate();
            break;
         }
         
         TP0 = 1;  // Diagnostic
         if(OpenLoopDwell == 0)
         {
            TMR0_run_state_timer = TIMEBASE_STARTUP_COUNT;
            SlowStartState = STARTINIT;
            OpenLoopFlag = 0;
            run_mode = ACCELERATE;
         }
         else
         {
            OpenLoopFlag = 1;
            TMR0_run_state_timer = OpenLoopDwell;
            SlowStartState = OPENLOOPRUN;
         }
         CCP4CON = CCP4CON_INIT;
         TMR1H = 0;
         TMR1L = 0;
         CCPR4H = TMR1_comm_time.bytes.high;
         CCPR4L = TMR1_comm_time.bytes.low;
         isr_state = commutate;
         CCP4IF = 0;
         TMR1ON = 1;
         CCP4IE = 1;
         PEIE=1;   
         GIE=1;
         break;

      case STEPDWELL:
         if(--TMR0_run_state_timer > 1)
            break;
         SlowStartState = SLOWSTEP;
         break;

      case OPENLOOPRUN:
         if(--TMR0_run_state_timer == 0)
         {
            TMR0_run_state_timer = TIMEBASE_STARTUP_COUNT;
            SlowStartState = STARTINIT;
            OpenLoopFlag = 0;
            run_mode = STALLDETECT;
         }
         break;

      default:
         SlowStartState = STARTINIT;
         break;
   }
}   

/************************************************************************
*                                                                       *
*      Function:       ControlStartup                                   *
*                                                                       *
*      Description:     check startup flags and timing                  *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*  Zero cross must be detected within the middle 25% of the             *
*  commutation period before the startup timer expires otherwise        *
*  the motor will be shut down.                                         *
*                                                                       *
*  Startup time is determined by the constant TIMEBASE_STARTUP_COUNT    *
*  which is defined in Main.h. Maximum startup time is 2.55 seconds.    *
*  This is called every main loop cycle but the TMR0_run_state_flag     *  
*  limits execution to every 10 ms as defined by the TimeBaseManager()  *
*   routine.                                                            *
*                                                                       *
*************************************************************************/

void ControlStartup(void) 
{
   if(--TMR0_run_state_timer == 0) 
   {
      // stable rotation will clear the startup_complete_flag.
      // if rotation is not stable by the time the startup is complete
      // then force a full reinitialization by setting the stop_flag..
      //startup_in_progress = 0;
      run_mode = STALLDETECT;
      if(!startup_complete_flag)
      {
         stop_flag=1;
      }   
   }
}

void SlowStartInit(void)
{
   SlowStartState = STARTINIT;
}
