/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : w.r.brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : 1947_Driver.h                              *
*                                                                       *
*     Other Files Required :                                            *
*     Tools Used: MPLAB GL : 8.30                                       *
*                 Compiler : Hi-Tech                                    *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************
*
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
// I/O definitions for F1 Evaluation board with PIC16LF1947                                           //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
*
*
*******************************************************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*******************************************************************************************************
*
*
*******************************************************************************************************
*
* Change History:
* Author               Date        Comment
* w.r.brown            2012.04.05  V 0.6  - Changed BORV_19 to BORV_LO to accomodate bit name change
* w.r.brown            2012.02.07  V 0.5  - Expanded comments for ADC channel definitions
* w.r.brown            2011.03.22  V 0.4  - Added battery voltage connection
* w.r.brown            2010.08.18  V 0.3  - Added drive indicators
* w.r.brown            2010.08.16  V 0.2  - Moved configuration fuse defines here from Main.h (formerly BLDC.h)
* w.r.brown            2010.08.13  V 0.1  - Derived from PIC16F1937.h V 0.2
*
*******************************************************************************************************/

#ifndef P1947_DRIVER_H
#define P1947_DRIVER_H

//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////// MCU Constants ///////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////// User Constants ///////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////// TimebaseManager times ////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// all TIMEBASE times must be expressed in even multiples of TIMEBASE_MS_PER_COUNT
// See Main.h

// TIMEBASE_STARTUP_ms = number of milliseconds allowed to achieve zero-cross lock
// (maximum ms is 255*TIMEBASE_MS_PER_COUNT)
#define  TIMEBASE_STARTUP_ms           2000

// time for continuous unstable condition to cause stall
// TIMER0_STALL_ms = number of milliseconds in out-of-lock condition to recognize stall
// (maximum ms is 255*TIMEBASE_MS_PER_COUNT)
#define  TIMER0_STALL_ms               1000
                                          
// number of milliseconds to dwell in warmup state
// (maximum ms is 255*TIMEBASE_MS_PER_COUNT)
#define  TIMEBASE_WARMUP_ms            400

// number of milliseconds between each motor supply voltage check
// (maximum ms is 255*TIMEBASE_MS_PER_COUNT)
#define  TIMEBASE_SUPPLY_ms            50

// number of milliseconds between each motor stall condition check
// (maximum ms is 255*TIMEBASE_MS_PER_COUNT)
#define  TIMEBASE_STALLCHECK_ms        500


//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////// Stall detection BEMF voltage  ///////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// R1 and R2 are series resistors from each motor terminal to ground
// ADC is connected to the junction of R1 and R2. The other end of R2 is connected to ground.
// Specify VDD_SUPPLY, BEMF_R1, and BEMF_R1 as 10 times the actual values
// Resistors are in K Ohms. Example: 5.6K is 56L.
#define  VDD_SUPPLY                 33L
#define  BEMF_R1                    470L
#define  BEMF_R2                    100L

// 10 times the BEMF voltage below which the motor will assumed to be stalled
// (i.e. for 1.2 Volts define limit as 12L)
#define  MINIMUM_BEMF_VOLTAGE       20L

#define  BEMF_LIMIT                 ((255L*BEMF_R2*MINIMUM_BEMF_VOLTAGE)/(VDD_SUPPLY*(BEMF_R1+BEMF_R2)))

//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////// Driver Board Configuration //////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// HIGH_SIDE_MODULATION - Define this variable for systems that modulate the high side driver.
//                        Comment out HIGH_SIDE_MODULATION so it remains undefined for systems 
//                        that modulate the low side driver.
//#define  HIGH_SIDE_MODULATION

// Device PIC16F1947 TQFP Pin assignments:
// 
// Pin #   PORT ID I/O Use  Name    Description
//------   ------- --- ---  ------- --------------------------
//  24       RA0    I  LCD  SEG33   5D/5E/5F/5A
//  23       RA1    I  LCD  SEG18   P5/5C/5G/5B
//  22       RA2    I  LCD  SEG34   6D/6E/6F/6A
//  21       RA3    I  AN3  RA3     Battery voltage
//  28       RA4    O       RA4     LED - Phase Z
//  27       RA5    I  AN4  RA5     Speed control input (new layout)
//  40       RA6    O       RA6     LED - BLDC A / Bipolar W 
//  39       RA7    O       RA7     LED - BLDC B / Bipolar X 
//  48       RB0    I  LCD  SEG30   3A/3F/3E/3D
//  47       RB1    I  LCD  SEG8    3B/3G/3C/P3
//  46       RB2    I  LCD  SEG9    4A/4F/4E/4D
//  45       RB3    I  LCD  SEG10   4B/4G/4C/P4
//  44       RB4    I  LCD  SEG11   F1/F3/F4/P6 
//  43       RB5    I  LCD  SEG29   F2/6C/6G/6B)
//  42       RB6    O       PGC     ICSP Clock (TestPoint 1)
//  37       RB7    O       PGD     ICSP Data  (TestPoint 2)
//  30       RC0    O       T1OSO   32.768 Oscillator
//  29       RC1    I       T1OSI   32.768 Oscillator
//  33       RC2    O       P1A     BLDC Low A / Bipolar Hi W  
//  34       RC3    I       SCK1    I2C Clock
//  35       RC4    I       SDA1    I2C Data
//  36       RC5    O       RC5     LED - BLDC C/Bipolar Y 
//  31       RC6    O       TX1     Serial Transmit
//  32       RC7    I       RX1     Serial Receive
//  58       RD0    I  LCD  SEG0    1A/1F/1E/1D
//  55       RD1    I  LCD  SEG1    1B/1G/1C/1P
//  54       RD2    I       RD2     Push button
//  53       RD3    I  LCD  SEG3    2A/2F/2E/2D
//  52       RD4    O       P3Ba    BLDC High C
//  51       RD5    O       P1Ca    Bipolar High X
//  50       RD6    O       P1Ba    BLDC High A / Bipolar Low W
//  49       RD7    I  LCD  SEG7    2B/2G/2C/P2
//   2       RE0    O       P2D     Bipolar Low Z
//   1       RE1    O       P2C     Bipolar High Z
//  64       RE2    O       P2B     BLDC High B / Bipolar Low Y
//  63       RE3    I  LCD  COM0    COM1
//  62       RE4    I  LCD  COM1    COM2      
//  61       RE5    I  LCD  COM2    COM3       
//  60       RE6    I  LCD  COM3    COM4       
//  59       RE7    O       P2Aa    BLDC Low B / Bipolar High Y       
//  18       RF0    I  AN16 C12IN0- Current sense       
//  17       RF1    I  LCD  SEG19   S8/S7/S6/S5       
//  16       RF2    0       RF2     C1OUT      
//  15       RF3    I  AN8  C12IN2- BEMF B / Bipolar current sense I2       
//  14       RF4    I  AN9  C2IN+   Current sense reference       
//  13       RF5    I  AN10 C12IN1- BEMF C       
//  12       RF6    I  AN11 C1IN+   BEMF Reference       
//  11       RF7    I  AN5  C12IN3- BEMF A / Bipolar current sense I1      
//   3       RG0    O       P3A     BLDC Low C       
//   4       RG1    I  LCD  SEG43   S1/S2/S3/S4       
//   5       RG2    I  LCD  SEG44   T1/T3/T2/T4              
//   6       RG3    O       RG3     UNIO              
//   8       RG4    O       P1D     Bipolar Low X              
//   7       RG5    I       MCLR/Vpp
//  10                      Vdd     Power supply
//  26                      Vdd     Power supply
//  38                      Vdd     Power supply
//  57                      Vdd     Power supply
//  19                      AVdd    Power supply
//   9                      Vss     Supply return
//  25                      Vss     Supply return
//  41                      Vss     Supply return
//  56                      Vss     Supply return
//  20                      AVss    Supply return
//
//////////////////////////////////////////////////////////////////////////////////////////
// PORTA (PORT) 
#define   TRISA_INIT       0b00101111
#define   PORTA_INIT       0b00000000
#define   TRISA_ERROR      0b11111111
#define   PORTA_ERROR      0b11111111

//////////////////////////////////////////////////////////////////////////////////////////
// PORTB (PORT)
#define   TRISB_INIT       0b00111111
#define   PORTB_INIT       0b00000000
#define   TRISB_ERROR      0b11111111
#define   PORTB_ERROR      0b11111111
   
//////////////////////////////////////////////////////////////////////////////////////////
// PORTC (PORT)
#define   TRISC_INIT       0b10011000
#define   PORTC_INIT       0b00000000
#define   TRISC_ERROR      0b11111111
#define   PORTC_ERROR      0b11111111

//////////////////////////////////////////////////////////////////////////////////////////
// PORTD (PORT)
#define   TRISD_INIT       0b10001111
#define   PORTD_INIT       0b00000000
#define   TRISD_ERROR      0b11111111
#define   PORTD_ERROR      0b11111111

//////////////////////////////////////////////////////////////////////////////////////////
// PORTE (PORT)
#define   TRISE_INIT       0b01111000
#define   PORTE_INIT       0b00000000
#define   TRISE_ERROR      0b11111111
#define   PORTE_ERROR      0b11111111

//////////////////////////////////////////////////////////////////////////////////////////
// PORTF (PORT)
#define   TRISF_INIT       0b11111011
#define   PORTF_INIT       0b00000000
#define   TRISF_ERROR      0b11111111
#define   PORTF_ERROR      0b11111111

//////////////////////////////////////////////////////////////////////////////////////////
// PORTG (PORT)
#define   TRISG_INIT       0b00100110
#define   PORTG_INIT       0b00000000
#define   TRISG_ERROR      0b11111111
#define   PORTG_ERROR      0b11111111

//////////////////////////////////////////////////////////////////////////////////////////
// APFCON Alternate pin functions

// Alternates: P1B, P1C, P2A, P3B
#define APFCON_INIT        0b01000111

//////////////////////////////////////////////////////////////////////////////////////////
// COMPARATORS

// redefine comparator definitions to ease selection between C1 and C2
// CMx is used for BEMF sense
// CMy is used for current sense
#define CMxCON0      CM1CON0
#define CMxCON1      CM1CON1
#define CxIE         C1IE
#define CxIF         C1IF
#define CxOUT        MC1OUT
#define COMPARATOR   CMxCON1

#define CMyCON0      CM2CON0
#define CMyCON1      CM2CON1
#define CyIE         C2IE
#define CyIF         C2IF
#define CyOUT        MC2OUT

//////////////////////////////////////////////////////////////////////////////////////////
// Comparator bit definitions
// 
// CMxCON0
#define CxON            0b10000000
#define CxOE            0b00100000
#define CxINV           0b00010000
#define CxFAST          0b00000100
#define CxHYST          0b00000010
#define CxSYNC          0b00000001

// CMxCON1
#define CxINTP          0b10000000
#define CxINTN          0b01000000
#define CxPIN           0b00000000
#define CxCDAC          0b00010000  
#define CxFVR           0b00100000
#define CxGND           0b00110000
#define CxIN0           0b00000000
#define CxIN1           0b00000001
#define CxIN2           0b00000010
#define CxIN3           0b00000011

//////////////////////////////////////////////////////////////////////////////////////////
// Comparator initializations

// BEMF comparator initialization
#ifdef HIGH_SIDE_MODULATION
// comparator ouput is inverted for high side modulation
#define  CMxCON0_INIT         CxON | CxOE | CxFAST | CxINV
#else
#define  CMxCON0_INIT         CxON | CxOE | CxFAST
#endif

#define  CMxCON1_INIT         SENSE_V_RISING

// Overcurrent sense comparator initialization
//#define CMyCON0_INIT    CxON | CxOE | CxINV  | CxPIN | CxIN3     

//////////////////////////////////////////////////////////////////////////////////////////
// ADC

#define  ANSELA_INIT          0b00101000
#define  ANSELE_INIT          0b00000000
#define  ANSELF_INIT          0b11111001
#define  ANSELG_INIT          0b00000000

// read AN5 for motor current or voltage on BLDC VU
#define CURRENT_CHANNEL1       5
// read AN8 for motor current or voltage on BLDC VV
#define CURRENT_CHANNEL2       8
// read AN9 for motor current or voltage on BLDC VW
#define CURRENT_CHANNEL       9
// read AN16 for motor current reference
#define CURRENT_REF_CHANNEL   16
// read AN5 for BLDC BEMF reference
#define BEMF_REFERENCE        11
//  read AN4 for speed control
#define  SPEED_CHANNEL        4
#define  ADCON0_SPEED         0b00010001
// read AN3 for battery voltage
#define  BATTERY_CHANNEL      3
#define  ADCON0_BATTERY       0b00001101
// raw ADC readings for battery levels
#define  BATTERY_FULL         0x1BF
#define  BATTERY_2_THIRDS     0x18F
#define  BATTERY_1_THIRD      0x15F
#define  BATTERY_EMPTY        0x0CC
#define  BATTERY_HYSTERESIS   0x00C

#ifdef HIGH_SIDE_MODULATION
// In state 2 read AN5 (BEMF A) for BEMF voltage
// this is when drive U-A is in the second period of high drive
// BEMF_STATE is compared to comm_state counter which is advanced by 1
// at end of the commutation switch call. So, while the motor is in state 2
// the comm_state counter is actually 3 in anticipation of the next state
#define  BEMF_STATE            3
#define  ADCON0_BEMF           0b00010101
#define  BEMF_DRIVE_BIT        MODULATE_U_BIT
#else
// In state 3 read AN8 (BEMF B) for BEMF voltage
// this is when drive V-B is in the first period of high drive
// BEMF_STATE is compared to comm_state counter which is advanced by 1
// at end of the commutation switch call. So, while the motor is in state 3
// the comm_state counter is actually 4 in anticipation of the next state
#define  BEMF_STATE            4
#define  ADCON0_BEMF           0b00100001
#define  BEMF_DRIVE_BIT        MODULATE_V_BIT
#endif

// BEMF_ACQUISITION_us is the time the flyback currents take to settle
#define  BEMF_ACQUISITION_us   45L

// Braking control measurements
// Reference is on RF6/AN11
#define  ADCON0_BEMF_REF       0b00101101
// Phase A is on RF7/AN5
#define  ADCON0_PHASE_A        0b00010101
// Phase B is on RF3/AN8
#define  ADCON0_PHASE_B        0b00100001
// Phase B is on RF5/AN10
#define  ADCON0_PHASE_C        0b00101001

// initialize in supply measurement mode
#define  ADCON0_INIT          ADCON0_SPEED   

#ifdef   FOSC_32_MHZ
//  TAD clock = Fosc/64 (2uS @ 32 MHz), Left justified, Vref-:Vss, Vref+:Vdd
#define  ADCON1_INIT          0b01100000
#endif

#ifdef   FOSC_8_MHZ
//  TAD clock = Fosc/16 (2uS @ 8 MHz)
#define  ADCON1_INIT          0b01010000
#endif

// ACQUISITION_TIME is the time between setting the GODONE bit and conversion start 
#define  ACQUISITION_TIME_us  4L

// BRAKE_ACQUISITION_us is the aquisition time after a BEMF channel change
#define  BRAKE_ACQUISITION_us   10L

//////////////////////////////////////////////////////////////////////////////////////////
// ECCP

// auto shutdown on C1OUT, drive output pins to 0
#define  ECCP1AS_INIT         0b00010000

// auto restart
#define  PWM1CON_INIT         0b10000000

// compare mode, reset Timer1 on match
#define  CCP4CON_INIT         0b00001011

// capture on first rising edge (not implemented)
//#define  CCP5CON_INIT         0b00000101

//////////////////////////////////////////////////////////////////////////////////////////
// SR Latch for hardware capture of zero cross events

// latch and Q output enable
#define  SRCON0_INIT          SRLEN | SRQEN

// set on C1 out high (zero cross event)
#define  SRCON1_INIT          SRSC1E

//////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////// Brushless Drive Definitions ///////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////
// Driver and comparator sense states

//    4|5|6|1|2|3|4|5|6|1|2|3|
//     | | |_|_| | | | |_|_| |
//  A _|_|/| | |\|_|_|/| | |\|
//    _| | | | |_|_| | | | |_|
//  B  |\|_|_|/| | |\|_|_|/| |
//     |_|_| | | | |_|_| | | |
//  C /| | |\|_|_|/| | |\|_|_|
//
//////////////// IF LOW SIDE MOD ////////////////////
// Phase               A        B        C
// PWM Side Drive   P1A/RC2  P2A/RE7  P3A/RG0
// Fixed Side Drive   RD6      RE2     RD4
// BEMF Sense       C12IN3-  C12IN2-  C12IN1-

///////////////// IF HIGH SIDE MOD //////////////////
// Phase               A        B        C
// PWM Side Drive   P1B/RD6  P2B/RE2  P3B/RD4
// Fixed Side Drive   RC2      RE7     RG0
// BEMF Sense       C12IN3-  C12IN2-  C12IN1-

#ifndef PSTRCON
#define PSTRCON PSTR1CON
#endif

// Direct drive writes the port in lieu of setting/clearing bits
// to minimize dead time between changes
#define DRIVE_PORT           LATE
#define DRIVE_INIT           0b00000000

// The PIC16LF1947 Evaluation board LEDs are seperate from driver outputs

#define PHASE_A_INDICATOR    LATA6
#define PHASE_B_INDICATOR    LATA7
#define PHASE_C_INDICATOR    LATC5
#define PHASE_U_INDICATOR    PHASE_A_INDICATOR
#define PHASE_V_INDICATOR    PHASE_B_INDICATOR
#define PHASE_W_INDICATOR    PHASE_C_INDICATOR

#ifdef HIGH_SIDE_MODULATION

// High side modulation bit control of drive pins
#define DRIVE_A_BIT          LATC2
#define DRIVE_B_BIT          LATE7
#define DRIVE_C_BIT          LATG0
#define DRIVE_U_BIT          DRIVE_A_BIT
#define DRIVE_V_BIT          DRIVE_B_BIT
#define DRIVE_W_BIT          DRIVE_C_BIT

// High side modulation PWM Drive steering
#define MODULATE_U_BIT       STR1B
#define MODULATE_V_BIT       STR2B
#define MODULATE_W_BIT       STR3B

#else

// Low side modulation bit control of drive pins
#define DRIVE_A_BIT          LATD6
#define DRIVE_B_BIT          LATE2
#define DRIVE_C_BIT          LATD4
#define DRIVE_U_BIT          DRIVE_A_BIT
#define DRIVE_V_BIT          DRIVE_B_BIT
#define DRIVE_W_BIT          DRIVE_C_BIT

// Low side modulation PWM Drive steering
#define MODULATE_U_BIT       STR1A
#define MODULATE_V_BIT       STR2A
#define MODULATE_W_BIT       STR3A

#endif

// Comparator steering
#define SENSE_A_RISING       CxINTP | CxPIN | CxIN3
#define SENSE_B_RISING       CxINTP | CxPIN | CxIN2
#define SENSE_C_RISING       CxINTP | CxPIN | CxIN1

#define SENSE_U_RISING       SENSE_A_RISING
#define SENSE_V_RISING       SENSE_B_RISING
#define SENSE_W_RISING       SENSE_C_RISING

#define SENSE_A_FALLING      SENSE_A_RISING
#define SENSE_B_FALLING      SENSE_B_RISING
#define SENSE_C_FALLING      SENSE_C_RISING
#define SENSE_U_FALLING      SENSE_A_FALLING
#define SENSE_V_FALLING      SENSE_B_FALLING
#define SENSE_W_FALLING      SENSE_C_FALLING

#define SENSE_A              SENSE_A_RISING
#define SENSE_B              SENSE_B_RISING
#define SENSE_C              SENSE_C_RISING

// HIGH_SIDE_MODULATION is determined at the top of this file.
#ifdef HIGH_SIDE_MODULATION
   // In high side modulation systems the BEMF is sensed as BEMF rises
   #define BEMF_FLAG rising_bemf_flag
#else
   // In low side modulation systems the BEMF is sensed as BEMF falls
   #define BEMF_FLAG !rising_bemf_flag
#endif

//////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////// Stepper Drive Definitions /////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// PWM Drive steering
// high side Bipolar modulation
#define MODULATE_1A_BIT      STR1A 
#define MODULATE_1C_BIT      STR1C
#define MODULATE_2A_BIT      STR2A 
#define MODULATE_2C_BIT      STR2C

// low side Bipolar modulation
#define MODULATE_1B_BIT      STR1B 
#define MODULATE_1D_BIT      STR1D
#define MODULATE_2B_BIT      STR2B 
#define MODULATE_2D_BIT      STR2D

// I/O Drive steering (modulation steering must be disabled)
// high side Bipolar drive 
#define DRIVE_1A_BIT         LATC2 
#define DRIVE_1C_BIT         LATD5
#define DRIVE_2A_BIT         LATE7 
#define DRIVE_2C_BIT         LATE1

// low side Bipolar drive 
#define DRIVE_1B_BIT         LATD6 
#define DRIVE_1D_BIT         LATG4
#define DRIVE_2B_BIT         LATE2 
#define DRIVE_2D_BIT         LATE0

//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////// Test Point Definitions //////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// use these points for scope reference and triggering

// startup and stable
#define TP0                  LATC0

// comm state 2 to end of blanking
#define TP1                  LATB6

// zero detect
#define TP2                  LATB7

// commutate to blanking
//#define TP3                  RC3

// run indicator
#define RunLED                LATA4

#define SW2                   RD2
#define PushButton            SW2
#define SpeedButton           RG5

//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////// EUSART Pin Definitions //////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

#define RX                    RC7
#define TX                    RC6

#endif

