/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : w.r.brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : EEData.c                                   *
*     Date                 : 2010/07/13                                 *
*                                                                       *
*     Other Files Required :                                            *
*     Tools Used: MPLAB GL : 8.14                                       *
*                 Compiler : Hi-Tech                                    *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************
*
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//   Routines to maintain non-volatile motor parameters.                                              //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
*
*
*******************************************************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*******************************************************************************************************
*
*
*******************************************************************************************************
*
* Change History:
* Author               Date        Comment
* w.r.brown            2011.08.24  V 0.7  - Added default stepper CCP_Val tables for Unipolar and Bipolar
* w.r.brown            2011.08.10  V 0.6  - Added stepper variables
* w.r.brown            2010.06.08  V 0.5  - Moved defines to EEData.h
* w.r.brown            2010.10.07  V 0.4  - Added braking parameters
* w.r.brown            2010.09.30  V 0.3  - Changed CommOffset and timing_balance from int time
*                                           to char degrees
* w.r.brown            2010.07.27  V 0.2  - added TimingBalance
* w.r.brown            2010.07.13  V 0.1  - original
*******************************************************************************************************/
#include <xc.h>
#include "Main.h"
#include "GenericTypeDefs.h"
#include "EEData.h"
#include "UNIO_support.h"
#include "BDC.h"
#include "Stepper.h"

extern mode_motor  motor_type;

//////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////// User adjustable variables/////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
    
// 8-bit data
extern volatile char low_startup_dutycycle;
extern volatile char startup_dutycycle;
extern volatile char hi_startup_dutycycle;
extern unsigned char Poles;
extern unsigned char blanking_count;
extern unsigned char SlowStartDwell;
extern unsigned char SlowStartSteps;
extern unsigned char OpenLoopDwell;
extern unsigned char timebase_duty_count;
extern unsigned char error_scale;
extern unsigned char speed_request_off;
extern unsigned char speed_request_on;
extern unsigned char upward_high_resolution_entry;
extern unsigned char upward_high_resolution_exit;
extern unsigned char downward_high_resolution_entry;
extern unsigned char downward_high_resolution_exit;
extern unsigned char speed_control_average_factor;
extern char          CommOffsetRequest;
extern char          TimingBalanceRequest;
extern unsigned char BrakeLevel;
extern unsigned char DutyScale;
extern unsigned char MinDuty;
extern unsigned char MicroStepSize;

// 16-bit data
extern volatile unsigned int startup_comm_time;
extern volatile unsigned int max_timer1_preset;

// 32-bit data
extern unsigned long SpeedScalingConst;
extern unsigned long SpeedConstTimer;

//////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////// Default Constants ////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

extern const unsigned char NumberOfPoles;
extern const unsigned char NumberOfPhases;
extern const unsigned int  Timer1Init;
extern const unsigned int  LowStartupDutyCycle;
extern const unsigned int  MediumStartupDutyCycle;
extern const unsigned int  HighStartupDutyCycle;
extern const unsigned char TimebaseDutyCount;
extern const unsigned char TimebaseSlowStep;
extern const unsigned char SlowSteps;
extern const unsigned char TimebaseOpenLoop;
extern const unsigned char BlankingCount;
extern const unsigned int  MaxTimer1Preset;
extern const unsigned char ErrorScale;
extern const char          TimingAdvanceDegrees;
extern const unsigned char SpeedRequestOff;
extern const unsigned char SpeedRequestOn;
extern const unsigned char UpwardHighResolutionEntry;
extern const unsigned char UpwardHighResolutionExit;
extern const unsigned char DownwardHighResolutionEntry;
extern const unsigned char DownwardHighResolutionExit;
extern const unsigned char SpeedControlAverageFactor;
extern const char          TimingBalanceDegrees;
extern const unsigned char DefaultBrakeLevel;
extern const unsigned char BipolarRequestOff;
extern const unsigned char BipolarRequestOn;
extern const unsigned char BipolarMinDuty;
extern const unsigned char BipolarDutyScale;
extern const unsigned char BipolarMicroStepSize;
extern const unsigned long BipolarSpeedConstTimer;
extern const unsigned long BipolarSpeedScalingConst;
extern const unsigned char UnipolarRequestOff;
extern const unsigned char UnipolarRequestOn;
extern const unsigned char UnipolarMinDuty;
extern const unsigned char UnipolarDutyScale;
extern const unsigned char UnipolarMicroStepSize;
extern const unsigned long UnipolarSpeedConstTimer;
extern const unsigned long UnipolarSpeedScalingConst;
extern const unsigned char BrushedRequestOff;
extern const unsigned char BrushedRequestOn;
extern const unsigned char Bipolar_CCP_Val[];
extern const unsigned char Unipolar_CCP_Val[];

//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
///////////////////////////////////////// CODE ///////////////////////////////////////////
//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void set_defaults(void)
{
   // set all NV variables to the default values
   
    timebase_duty_count             = TimebaseDutyCount;
    switch (motor_type)
    {
        default:
        //case BLDC:
            Poles                           = NumberOfPoles;
            low_startup_dutycycle           = LowStartupDutyCycle;
            startup_dutycycle               = MediumStartupDutyCycle;
            hi_startup_dutycycle            = HighStartupDutyCycle;
            startup_comm_time               = Timer1Init;
            max_timer1_preset               = MaxTimer1Preset;
            CommOffsetRequest               = TimingAdvanceDegrees;
            blanking_count                  = BlankingCount;
            SlowStartSteps                  = SlowSteps;
            SlowStartDwell                  = TimebaseSlowStep;
            OpenLoopDwell                   = TimebaseOpenLoop;
            error_scale                     = ErrorScale;
            upward_high_resolution_entry    = UpwardHighResolutionEntry;
            upward_high_resolution_exit     = UpwardHighResolutionExit;
            downward_high_resolution_entry  = DownwardHighResolutionEntry;
            downward_high_resolution_exit   = DownwardHighResolutionExit;
            speed_control_average_factor    = SpeedControlAverageFactor;
            TimingBalanceRequest            = TimingBalanceDegrees;
            BrakeLevel                      = DefaultBrakeLevel;
   
            speed_request_off               = SpeedRequestOff;
            speed_request_on                = SpeedRequestOn;
            break;
        case BDC:    
            speed_request_off               = BrushedRequestOff;
            speed_request_on                = BrushedRequestOn;
            break;
        case UNIPOLAR:    
            speed_request_off               = UnipolarRequestOff;
            speed_request_on                = UnipolarRequestOn;
            MinDuty                         = UnipolarMinDuty;
            DutyScale                       = UnipolarDutyScale;
            MicroStepSize                   = UnipolarMicroStepSize;
            SpeedConstTimer                 = UnipolarSpeedConstTimer;
            SpeedScalingConst               = UnipolarSpeedScalingConst;
            ReadDefaultStepperTable(Unipolar_CCP_Val);
            break;
        case BIPOLAR:    
            speed_request_off               = BipolarRequestOff;
            speed_request_on                = BipolarRequestOn;
            MinDuty                         = BipolarMinDuty;
            DutyScale                       = BipolarDutyScale;
            MicroStepSize                   = BipolarMicroStepSize;
            SpeedConstTimer                 = BipolarSpeedConstTimer;
            SpeedScalingConst               = BipolarSpeedScalingConst;
            ReadDefaultStepperTable(Bipolar_CCP_Val);
            break;
    }
    NV_Variables_Save();
}
//------------------------------------------------------------------------------
void NV_Variables_Read(void)
{
    // load all the non-volatile variables from EEPROM
   
    timebase_duty_count            = eeprom_read(nvTimebaseDutyCount);
    
    switch (motor_type)
    {
        default:
            //case BLDC:
            Poles                          = eeprom_read(nvPoles);
            // production test with erased EEMemory will set defaults
            if (Poles < 2 || Poles > 100)
                set_defaults();
            else
            {
                low_startup_dutycycle          = eeprom_read(nvLowStartupDC);
                startup_dutycycle              = eeprom_read(nvStartupDC);
                hi_startup_dutycycle           = eeprom_read(nvHiStartupDC);
                startup_comm_time              = eeprom_read(nvStartupCommTimeHigh)<<8;
                startup_comm_time             += eeprom_read(nvStartupCommTimeLow);
                max_timer1_preset              = eeprom_read(nvMaxTimer1PresetHigh)<<8;
                max_timer1_preset             += eeprom_read(nvMaxTimer1PresetLow);
                //CommOffset                     = eeprom_read(nvCommOffsetHigh)<<8;
                CommOffsetRequest              = eeprom_read(nvCommOffset);
                blanking_count                 = eeprom_read(nvBlankingCount);
                SlowStartSteps                 = eeprom_read(nvSlowStartSteps);
                SlowStartDwell                 = eeprom_read(nvSlowStartDwell);
                OpenLoopDwell                  = eeprom_read(nvOpenLoopDwell);
                error_scale                    = eeprom_read(nvErrorScale);
                upward_high_resolution_entry   = eeprom_read(nvUpwardHighResolutionEntry);
                upward_high_resolution_exit    = eeprom_read(nvUpwardHighResolutionExit);
                downward_high_resolution_entry = eeprom_read(nvDownwardHighResolutionEntry);
                downward_high_resolution_exit  = eeprom_read(nvDownwardHighResolutionExit);
                speed_control_average_factor   = eeprom_read(nvSpeedControlAverageFactor);
                //timing_balance                 = eeprom_read(nvTimingBalanceHigh)<<8;
                TimingBalanceRequest           = eeprom_read(nvTimingBalance);
                BrakeLevel                     = eeprom_read(nvBrakeLevel);
        
                speed_request_off              = eeprom_read(nvSpeedRequestOff);
                speed_request_on               = eeprom_read(nvSpeedRequestOn);
            }
            break;
        case BDC:
   	        ReadBDCParameters();
            break;
        case UNIPOLAR:    
        case BIPOLAR:    
   	        // failed checksum will cause UNDEFINED motor type and indicate fault
   	        if(!ReadStepperParameters() || !ReadStepperTable())
   	            motor_type = UNDEFINED;
            break;
    }
}
//------------------------------------------------------------------------------
void NV_Variables_Save(void)
{
   // Re-write all non-volatile variables to EEPROM
   
    switch (motor_type)
    {
        default:
        //case BLDC:
            eeprom_write(nvPoles, Poles);
            eeprom_write(nvLowStartupDC, low_startup_dutycycle);
            eeprom_write(nvStartupDC, startup_dutycycle);
            eeprom_write(nvHiStartupDC, hi_startup_dutycycle);
            eeprom_write(nvStartupCommTimeHigh,((startup_comm_time>>8) & 0xFF));
            eeprom_write(nvStartupCommTimeLow,(startup_comm_time & 0xFF) );
            eeprom_write(nvMaxTimer1PresetHigh,((max_timer1_preset>>8) & 0xFF));
            eeprom_write(nvMaxTimer1PresetLow,(max_timer1_preset & 0xFF));
            //eeprom_write(nvCommOffsetHigh,((CommOffset>>8) & 0xFF));
            eeprom_write(nvCommOffset,CommOffsetRequest);
            eeprom_write(nvBlankingCount, blanking_count);
            eeprom_write(nvSlowStartSteps,SlowStartSteps);
            eeprom_write(nvSlowStartDwell,SlowStartDwell);
            eeprom_write(nvOpenLoopDwell,OpenLoopDwell);
            eeprom_write(nvTimebaseDutyCount,timebase_duty_count);
            eeprom_write(nvErrorScale,error_scale);
            eeprom_write(nvUpwardHighResolutionEntry,upward_high_resolution_entry);
            eeprom_write(nvUpwardHighResolutionExit,upward_high_resolution_exit);
            eeprom_write(nvDownwardHighResolutionEntry,downward_high_resolution_entry);
            eeprom_write(nvDownwardHighResolutionExit,downward_high_resolution_exit);
            eeprom_write(nvSpeedControlAverageFactor,speed_control_average_factor);
            //eeprom_write(nvTimingBalanceHigh,((timing_balance>>8) & 0xFF));
            eeprom_write(nvTimingBalance,TimingBalanceRequest);
            eeprom_write(nvBrakeLevel, BrakeLevel);
   
            eeprom_write(nvSpeedRequestOff,speed_request_off);
            eeprom_write(nvSpeedRequestOn,speed_request_on);
            break;
       case BDC:    
            InitUNIO();    
            WriteUNIOByte(nvSpeedRequestOff,speed_request_off);
            WriteUNIOByte(nvSpeedRequestOn,speed_request_on);
            RestoreUNIO();
            break;
       case UNIPOLAR:    
       case BIPOLAR:
            WriteStepperParameters();
            WriteStepperTable();    
            break;
   }
}
//------------------------------------------------------------------------------
void NV_SPBRG_Read(void)
{
   SPBRG = eeprom_read(nvSPBRG);
}
//------------------------------------------------------------------------------
void NV_SPBRG_Save(void)
{
   char SPBRG_Value;
   
   SPBRG_Value = SPBRG; 
   eeprom_write(nvSPBRG, SPBRG_Value);
}   
      
