/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : w.r.brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : PC-ADC_SpeedManager_Combo.c                *
*     Date                 : 2011/03/23                                 *
*                                                                       *
*     Other Files Required :                                            *
*     Tools Used: MPLAB GL : 8.14                                       *
*                 Compiler : Hi-Tech                                    *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************
*
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//   Combined closed and open loop PC speed manager.                                                  //
//   Request for motor speed is determined by input from PC via USART or pot on board.                //
//   The PC has direct access to GPR space and will change speedrequest directly.                     //
//   The speed manager will take the largest of the two requests.                                     //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
*
*
*******************************************************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*******************************************************************************************************
*
*
*******************************************************************************************************
*
* Change History:
* Author               Date        Comment
* w.r.brown            2011.08.09  V 0.2  - added stepper speed control
* w.r.brown            2011.03.23  V 0.1  - derived from PC_SpeedManager_Combo.c
*******************************************************************************************************/
#include <xc.h>
#include "Main.h"
#include "Monitor.h"
#include "UartInterface.h"
#include "SpeedManager.h"
#include "Init.h"

const char rsSpeedVersion[] = "PC-ADC 0.2 ";

extern volatile doublebyte InputData;
volatile bit run_arm_flag;
bit new_speed;
bit CCPValReady;

extern bit stop_flag;
extern bit startup_complete_flag;
extern bit run_flag;
extern bit TMR0_duty_flag;
extern char TMR0_duty_timer;
extern unsigned char ramped_speed;
extern bit OpenLoopFlag;
extern bit SingleStep;
extern bit SpeedReady;
extern bit OffsetReady;
extern int CommOffset;
extern int timing_balance;
extern bit BalanceReady;
extern unsigned char speed_request_off;
extern unsigned char speed_request_on;
extern unsigned char timebase_duty_count;
extern unsigned char stepSize;
extern int pot_value;

extern unsigned char DutyScale;
extern unsigned char MinDuty;
extern unsigned long SpeedScalingConst;
extern mode_motor  motor_type;

unsigned char speedrequest;
extern doublebyte TMR1_comm_time;

bit LastSpeedButton;

/************************************************************************
*                                                                       *
*      Function: 	     SpeedManager                                   *
*                                                                       *
*      Description:    Determine desired speed from                     *
*                      value entered directly into speedrequest         *
*                      or ADC reading taken outside this routine.       *
*                      Speed is controlled by changing the duty cyle    *
*                      of ECCP1.                                        *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

void SpeedManager(void) 
{
    unsigned char pot_byte;
    unsigned char speed;
    unsigned int duty_cycle;
    unsigned long commTime;
    
    ServiceCommunicator();

    if(!TMR0_duty_flag) return;            // hold off for 10 ms

	TMR0_duty_flag=0;
	if(--TMR0_duty_timer) return;          // wait until timer times out

	TMR0_duty_timer = timebase_duty_count;  // reset timer

    if(OpenLoopFlag)
    {
        // detect and debounce button push action
        if (SpeedButton == 0)
            if (LastSpeedButton == 1)
                SingleStep = 1;
        LastSpeedButton = SpeedButton;
    }
    else
    {
        pot_byte = pot_value>>2;
      
        if(SpeedReady)        // process PC only when new data has been entered
        {
            speedrequest = InputData.bytes.high;
            SpeedReady = 0;    // clear handshake flag
        }
      
        // speedrequest is largest of pot_value or InputData
        if(pot_byte > speedrequest)
            speed = pot_byte;
        else
            speed = speedrequest;   

        // stop-run hysterisis: stop when below low threshold   
   	    if(speed < speed_request_off)
   	    {
      	    if(run_flag) stop_flag = 1;        // this forces a system reset stop during run
      	    // if some event other than speed request causes the stop_flag to be set
      	    // then the run_arm_flag will not be rearmed until the speed request goes below
      	    // the run stop threshold. This prevents faults from restarting unexpectedly.
      	    run_arm_flag = 1;
      	    // continue to process Commutation offset and balance changes when stopped
      	    if(OffsetReady)
      	    {
                CommOffset = InputData.word;
                OffsetReady = 0;
            }
            if(BalanceReady)
            {
                timing_balance = InputData.word;
                BalanceReady = 0;
            }
      	    return;
        }
        // stop-run hysterisis: don't start until above high threshold
        if(speed > speed_request_on)
            //if(run_arm_flag)  // uncomment this line to prevent auto-restart after fault
                run_flag = 1;
      
   	    if(!startup_complete_flag) return;     // do not start to accelerate until startup is complete
        run_arm_flag = 0;  // this prevents auto-restart
      
   	    // ramp up or down to the requested speed setting
   	    // NOTE: if the speedrequest-average sample size is sufficently large then this
   	    //       ramping function can be eliminated.
   	    if(speed > ramped_speed) ramped_speed++;
   	    if(speed < ramped_speed) ramped_speed--;
   	  
   	    switch (motor_type)
   	    {
       	    case BDC:
            case BLDC:
                // set the motor voltage PWM by accessing values in a table
                // indexed by the global variable ramped_speed
           	    GetCCPVal(ramped_speed);
   	            break;
   	        default:
   	        //case BIPOLAR:
   	        //case UNIPOLAR:
                duty_cycle = ramped_speed*DutyScale;
                speed = (duty_cycle>>8) + MinDuty;
                // limit speed to maximum duty cycle
                //if(speed > PR2) speed = PR2 + 1;
                if(speed != EEDATL)
                {
                	EEDATL = speed;
                	// set flag to signal ISR to reload the duty cycle registers
                	CCPValReady = 1;
                	// change commutation period in response to speed change
                	commTime = ((SpeedScalingConst>>stepSize)/ramped_speed);
                	if(commTime > 0x0FFFF) commTime = 0xFFFF;                	    
            	 	TMR1_comm_time.word = (unsigned int)commTime;
            		// set flag to signal ISR to reload the period registers
            	    new_speed = 1;
                }
                break;
        }
    }     
}

void SpeedManagerInit(void)
{
    // This version of the SpeedManager uses inputs from the PC Monitor program
    // so we initialze the monitor here. This allows for changing to a non-PC
    // speed manager by simply replacing this file with another and removing all
    // monitor files from the project list.
   MonitorInit();
}
    