/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : I. Boholteanu                              *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : Bipolar_Interrupts_Plain.c                 *
*     Date                 : 2011/03/30                                 *
*                                                                       *
*     Tools Used: MPLAB GL : 8.14                                       *
*                 Compiler : Hi-Tech                                    *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************
*
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//   Plain vanilla interrupts with two states: Zero_detect and Commutate.                             //
//   This version also includes dynamic blanking.                                                     //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
*
*
*********************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************
*
* Change History:
* Author               Date        Comment
* w.r.brown           2011.08.22  V 2.1 - Added WAVE commutation selection
* w.r.brown           2011.07.21  V 2.0 - Complete overhaul
* i. boholteanu       2011.03.30  V 1.0 - first revision
*******************************************************************************************************/
#include <xc.h>
#include "Main.h"
#include "GenericTypeDefs.h"
#include "Stepper.h"

extern doublebyte TMR1_comm_time;
extern unsigned char comm_state;
extern unsigned int stepCount;
extern unsigned char stepSize;
extern unsigned char MicroStepSize;
unsigned int stepInc;
extern bit new_speed;
extern bit StepReady;
extern bit ReverseDirection;
extern mode_step step_mode;

// variables for open loop

extern volatile doublebyte InputData;
extern bit OpenLoopFlag;
extern bit SingleStep;
extern bit SpeedReady;


//////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////// Private Prototypes ////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

void FullStepCommutate(void);
void HalfStepCommutate(void);
void MicroStepCommutate(void);


////////////////////////////////////////////////////////////////////////////////////////////////////////
//    Motor drive is controlled by interrupts. There is only one type of interrupt:                   //
//    Timer1 compare.                                                                                 //
//                                                                                                    //
//    Timer1 interrupts:                                                                              //
//       Commutation interrupt - Motor drive is changed to the next commutation phase.                //
                                                                                                      //
////////////////////////////////////////////////////////////////////////////////////////////////////////
void bipolar_interrupt_handler(void) 
{ 
    if(step_mode == MICRO_STEP)
    {
        if(StepReady)
        {
		    stepSize = MicroStepSize;
		    stepInc = LOOK_UP_TABLE_SIZE>>stepSize;
		    stepCount = 0;
		    EEDATL = 0;
		    StepReady = 0;
		}

        if(ReverseDirection)
            stepCount -= stepInc;
        else
            stepCount += stepInc;
        stepCount %= 1024;

	    GetStep();
        MicroStepCommutate();
    }
    else
    {
        GetStep();
    }

    if(step_mode == FULL_STEP || step_mode == WAVE)
    {
        if(ReverseDirection)
        {
            if(--comm_state < 1) comm_state = 4;
        }
        else
        {
            if(++comm_state > 4) comm_state = 1;
        }
        if(step_mode == WAVE)
            FullStepCommutate();
        else
            MicroStepCommutate();
    }
   
    if(step_mode == HALF_STEP)
    {
        if(ReverseDirection)
        {
            if(--comm_state < 1) comm_state = 8;
        }
        else
        {
            if(++comm_state > 8) comm_state = 1;
        }
        
        HalfStepCommutate();
    }
        
    // prevent update when InputData is in transition
    // PC sets SpeedReady flag to indicate that the InputData is stable
    // 
    if (OpenLoopFlag)
        if(SpeedReady)
            {
                if(InputData.word == 0) InputData.word = 1;
                TMR1_comm_time.word = -InputData.word;
                TMR1_comm_time.word = ((TMR1_comm_time.word>>2)&0x3FFF);
                SpeedReady = 0;
                new_speed = 1;
            }

    // coordinate speed changes with interrupt execution
    if(new_speed)
    {
    	CCPR4H = TMR1_comm_time.bytes.high;
    	CCPR4L = TMR1_comm_time.bytes.low;
    	new_speed = 0;
    }

    LATA6 = !LATA6;

    CCP4IF = 0;
} 

////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//                                     Bipolar commutations                                           //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
void FullStepCommutate(void)   
{
	switch (comm_state)
	{
		case 1: // A forward
		 
			// forward progression
			DRIVE_2B_BIT = 0;	
			MODULATE_2C_BIT = 0;
            //reverse progression
			DRIVE_2D_BIT = 0;	
			MODULATE_2A_BIT = 0;
            // A Forward
			DRIVE_1D_BIT = 1;	
			MODULATE_1A_BIT = 1;
			break;
				
		case 2:  // B forward
			// forward progression
			DRIVE_1D_BIT = 0;	
			MODULATE_1A_BIT = 0;
            //reverse progression
			DRIVE_1B_BIT = 0;	
			MODULATE_1C_BIT = 0;
            // B Forward
			DRIVE_2D_BIT = 1;	
			MODULATE_2A_BIT = 1;
			break;	
				
		case 3:  // A reverse
			// forward progression
			DRIVE_2D_BIT = 0;	
			MODULATE_2A_BIT = 0;
            //reverse progression
			DRIVE_2B_BIT = 0;	
			MODULATE_2C_BIT = 0;
            // A Reverse
			DRIVE_1B_BIT = 1;	
			MODULATE_1C_BIT = 1;
			break;
				
		case 4: // B reverse
			// forward progression
			DRIVE_1B_BIT = 0;	
			MODULATE_1C_BIT = 0;
            //reverse progression
			DRIVE_1D_BIT = 0;	
			MODULATE_1A_BIT = 0;
			// B Reverse
			DRIVE_2B_BIT = 1;	
			MODULATE_2C_BIT = 1;
			break;
				
		default:	//comm_state = 1;
			break;											
	}
}	
	
void HalfStepCommutate(void)   
{
	switch (comm_state)
	{
		case 1: // A forward
			// B reverse off: forward progression
			DRIVE_2B_BIT = 0;	
			MODULATE_2C_BIT = 0;
            // B forward off: reverse progression
			DRIVE_2D_BIT = 0;	
			MODULATE_2A_BIT = 0;
            // A Forward
			DRIVE_1D_BIT = 1;	
			MODULATE_1A_BIT = 1;
			break;

		case 2: // A & B forward
            // A Forward
			DRIVE_1D_BIT = 1;	
			MODULATE_1A_BIT = 1;
            // B Forward
			DRIVE_2D_BIT = 1;	
			MODULATE_2A_BIT = 1;
			break;
				
		case 3:  // B forward
			// A forward off: forward progression
			DRIVE_1D_BIT = 0;	
			MODULATE_1A_BIT = 0;
            // A reverse off: reverse progression
			DRIVE_1B_BIT = 0;	
			MODULATE_1C_BIT = 0;
            // B Forward
			DRIVE_2D_BIT = 1;	
			MODULATE_2A_BIT = 1;
			break;	
				
		case 4:  // B forward, A reverse
            // B Forward
			DRIVE_2D_BIT = 1;	
			MODULATE_2A_BIT = 1;
            // A Reverse
			DRIVE_1B_BIT = 1;	
			MODULATE_1C_BIT = 1;
			break;	
				
		case 5:  // A reverse
            // B reverse off: reverse progression
			DRIVE_2B_BIT = 0;	
			MODULATE_2C_BIT = 0;
            // B forward off: forward progression
			DRIVE_2D_BIT = 0;	
			MODULATE_2A_BIT = 0;
            // A Reverse
			DRIVE_1B_BIT = 1;	
			MODULATE_1C_BIT = 1;
			break;
				
		case 6:  // B & A reverse
            // A Reverse
			DRIVE_1B_BIT = 1;	
			MODULATE_1C_BIT = 1;
            // B Reverse
			DRIVE_2B_BIT = 1;	
			MODULATE_2C_BIT = 1;
			break;
				
		case 7:  // B reverse
			// A reverse off: forward progression
			DRIVE_1B_BIT = 0;	
			MODULATE_1C_BIT = 0;
            // A forward off: reverse progression
			DRIVE_1D_BIT = 0;	
			MODULATE_1A_BIT = 0;
            // B Reverse
			DRIVE_2B_BIT = 1;	
			MODULATE_2C_BIT = 1;
			break;
				
		case 8:  // B reverse, A forward
            // B Reverse
			DRIVE_2B_BIT = 1;	
			MODULATE_2C_BIT = 1;
            // A Forward
			DRIVE_1D_BIT = 1;	
			MODULATE_1A_BIT = 1;
			break;
				
		default:	//comm_state = 1;
			break;											
	}

}	

void MicroStepCommutate(void)   
{
	switch (comm_state)
	{
		case 1: // A & !B
		 
			// forward: remove !A
			DRIVE_1B_BIT = 0;	
			MODULATE_1C_BIT = 0;
            //reverse: remove B
			DRIVE_2D_BIT = 0;	
			MODULATE_2A_BIT = 0;
            // A Forward
			DRIVE_1D_BIT = 1;	
			MODULATE_1A_BIT = 1;
			// B Reverse
			DRIVE_2B_BIT = 1;	
			MODULATE_2C_BIT = 1;
			break;
				
		case 2:  // A & B
			// forward: remove !B
			DRIVE_2B_BIT = 0;	
			MODULATE_2C_BIT = 0;
            //reverse: remove !A
			DRIVE_1B_BIT = 0;	
			MODULATE_1C_BIT = 0;
            // A Forward
			DRIVE_1D_BIT = 1;	
			MODULATE_1A_BIT = 1;
            // B Forward
			DRIVE_2D_BIT = 1;	
			MODULATE_2A_BIT = 1;
			break;	
				
		case 3:  // !A & B
			// forward: remove A
			DRIVE_1D_BIT = 0;	
			MODULATE_1A_BIT = 0;
            //reverse: remove !B
			DRIVE_2B_BIT = 0;	
			MODULATE_2C_BIT = 0;
            // A Reverse
			DRIVE_1B_BIT = 1;	
			MODULATE_1C_BIT = 1;
            // B Forward
			DRIVE_2D_BIT = 1;	
			MODULATE_2A_BIT = 1;
			break;
				
		case 4: // !A & !B
			// forward: remove B
			DRIVE_2D_BIT = 0;	
			MODULATE_2A_BIT = 0;
            //reverse: remove A
			DRIVE_1D_BIT = 0;	
			MODULATE_1A_BIT = 0;
            // A Reverse
			DRIVE_1B_BIT = 1;	
			MODULATE_1C_BIT = 1;
			// B Reverse
			DRIVE_2B_BIT = 1;	
			MODULATE_2C_BIT = 1;
			break;
				
		default:	//comm_state = 1;
			break;											
	}
}	
