/************************************************************************
*                                                                       *
*     Project              : F1 LV Evaluation Demo                      *
*                                                                       *
*     Author               : w.r.brown                                  *
*                                                                       *
*     Company              : Microchip Technology Incorporated          *
*     Filename             : Stepper.c                                  *
*     Date                 : 2011/08/04                                 *
*                                                                       *
*     Other Files Required :                                            *
*     Tools Used: MPLAB GL : 8.70                                       *
*                 Compiler : Hi-Tech                                    *
*                 Assembler:                                            *
*                 Linker   :                                            *
*                                                                       *
*************************************************************************
*
////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
//    Stepper motor add-on board routines common to bipoler and unipolar motors for use with          //
//    F1 LV Evaluation board.                                                                         //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
*
*******************************************************************************************************
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*******************************************************************************************************
*
*
********************************************************************************************************
* Change History:
* Author               Date           Comment
*
********************************************************************************************************
*/

#include <xc.h>
#include "Main.h"
#include "GenericTypeDefs.h"
#include "EEData.h"
#include "UNIO_support.h"
#include "Stepper.h"

//extern bit ReverseDirection;
extern bit startup_complete_flag;
extern unsigned char speed_request_off;
extern unsigned char speed_request_on;
extern unsigned char DutyScale;
extern unsigned char MinDuty;
extern unsigned char MicroStepSize;
extern unsigned long SpeedScalingConst;
extern unsigned long SpeedConstTimer;

unsigned int stepCount;
unsigned char CCP_Val[LOOK_UP_TABLE_SIZE]; 
extern unsigned char comm_state;
//extern doublebyte TMR1_comm_time;
extern bit CCPValReady;
extern mode_step step_mode;
extern mode_run run_mode;

/************************************************************************
*                                                                       *
*      Function:       void ReadStepperParameters(void)                 *
*                                                                       *
*      Description:    Reads user definable stepper motor parameters    *
*                      from UNIO memory                                 *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

BOOL ReadStepperParameters(void) 
{
    unsigned int checksum = 0;
    BOOL return_value = TRUE;
    
	// adjust Timer1 and CCP1 for UNIO control
	InitUNIO();
	// read and check all parameters
	checksum = ReadUNIOWord(nvParametersChecksum);
	speed_request_off = ReadUNIOByte(nvSpeedRequestOff);
	checksum -= speed_request_off;
	speed_request_on = ReadUNIOByte(nvSpeedRequestOn);
	checksum -= speed_request_on;
	DutyScale = ReadUNIOByte(nvDutyScale);
	checksum -= DutyScale;
	MinDuty = ReadUNIOByte(nvMinDuty);
	checksum -= MinDuty;
	MicroStepSize = ReadUNIOByte(nvMicroStepSize);
	checksum -= MicroStepSize;
    checksum -= ReadUNIOLong(nvSpeedConstTimer, &SpeedConstTimer);
    checksum -= ReadUNIOLong(nvSpeedScalingConst, &SpeedScalingConst);
	// restore Timer1 and CCP1
	RestoreUNIO();

	if (checksum != 0) return_value = FALSE;
	return return_value;
}
/************************************************************************
*                                                                       *
*      Function:       void ReadStepperTable(void)                      *
*                                                                       *
*      Description:    Reads user definable microstep CCP table values  *
*                      from UNIO memory                                 *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

BOOL ReadStepperTable(void) 
{
    unsigned int checksum = 0;
    BOOL return_value = TRUE;
    
	// adjust Timer1 and CCP1 for UNIO control
	InitUNIO();
	checksum = ReadUNIOWord(nvTableChecksum);
	// restore Timer1 and CCP1
	RestoreUNIO();
    checksum += ReadUNIOBlock(CCP_Val,nvCCP_Val,LOOK_UP_TABLE_SIZE);

	if (checksum != 0) return_value = FALSE;
	return return_value;
}
/************************************************************************
*                                                                       *
*      Function:       void WriteStepperParameters(void)                *
*                                                                       *
*      Description:    Writes stepper motor parameters in SRAM to       *
*                      UNIO memory.                                     *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

void WriteStepperParameters(void) 
{
    unsigned int checksum;

    // set factory defaults for motor parameters

    InitUNIO();			

	checksum = speed_request_off;
	checksum += speed_request_on;
	checksum += MinDuty;
	checksum += DutyScale;
	checksum += MicroStepSize;
	
	WriteUNIOByte(nvSpeedRequestOff, speed_request_off);
	WriteUNIOByte(nvSpeedRequestOn, speed_request_on);
	WriteUNIOByte(nvMinDuty, MinDuty);
	WriteUNIOByte(nvDutyScale, DutyScale);
	WriteUNIOByte(nvMicroStepSize, MicroStepSize);
	checksum += WriteUNIOLong(nvSpeedConstTimer, SpeedConstTimer);
	checksum += WriteUNIOLong(nvSpeedScalingConst, SpeedScalingConst);

	WriteUNIOWord(nvParametersChecksum,checksum);

    RestoreUNIO();
}

/************************************************************************
*                                                                       *
*      Function:       void WriteStepperTable(void)                     *
*                                                                       *
*      Description:    Writes microstep CCP SRAM table values to        *
*                      UNIO memory.                                     *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

void WriteStepperTable(void) 
{
    unsigned int checksum;

    // transfer SRAM table to UNIO memory
    checksum = WriteUNIOBlock(CCP_Val,nvCCP_Val,LOOK_UP_TABLE_SIZE);

    InitUNIO();			
	WriteUNIOWord(nvTableChecksum,checksum);
    RestoreUNIO();
}

/************************************************************************
*                                                                       *
*      Function:       void ReadDefaultStepperTable(void)               *
*                                                                       *
*      Description:    Reads default microstep CCP table values         *
*                      into CCP SRAM memory.                            *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

void ReadDefaultStepperTable(const unsigned char* Source) 
{
    const unsigned char* src;
    unsigned char* dest;
    int i;
      
    src = Source;
    dest = CCP_Val;

    // transfer default values into SRAM
    for(i=0;i<LOOK_UP_TABLE_SIZE;i++)
        *dest++ = *src++;
}

/************************************************************************
*                                                                       *
*      Function:       void SetCCPVal(unsigned char duty1, unsigned char duty2)
*                                                                       *
*      Description:    Sets duty cycle for CCP1 and CCP2                *
*                      Duty cycle values are 8-bit numbers where the    *
*                      most significant bit is bit 6 of CCPRxL and      *
*                      the least significant bit is DCxB1               *
*                                                                       *
*      Parameters:                                                      *
*                   duty1 : 8-bit duty cycle value for CCP1             *
*                   duty2 : 8-bit duty cycle value for CCP2             *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

void SetCCPVal(unsigned char duty1, unsigned char duty2)
{    
     DC1B1  = duty1 & 0x01;
     CCPR1L = (duty1 >> 1);
	
     DC2B1  = duty2 & 0x01;
     CCPR2L = (duty2 >> 1);
} 

/************************************************************************
*                                                                       *
*      Function:       void GetStep(void)                               *
*                                                                       *
*      Description:    Determine the CCP duty cycle from the step_mode  *
*                      and step count then set the comm_state and       *
*                      CCPRxL registers.                                *
*                                                                       *
*      Parameters:                                                      *
*      Return value:                                                    *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/
void GetStep(void)
{
    unsigned char indx, nindx;
    	
    if(step_mode == MICRO_STEP)
    {		
        indx = stepCount % LOOK_UP_TABLE_SIZE;
        nindx = (LOOK_UP_TABLE_SIZE-1) - indx;
    	indx = CCP_Val[indx];
    	nindx = CCP_Val[nindx];
    	
    	if(stepCount < LOOK_UP_TABLE_SIZE)
    	{
            SetCCPVal(indx, nindx);
    	   	comm_state = 1;
    	}
    	// [90..180] deg - 2nd quadrant
    	else if(stepCount < 2*LOOK_UP_TABLE_SIZE)
    	{
            SetCCPVal(nindx, indx);
    	   	comm_state = 2;
    	}
    	// [180..270] deg - 3rd quadrant
    	else if(stepCount < 3*LOOK_UP_TABLE_SIZE)
    	{
            SetCCPVal(indx, nindx);
    	   	comm_state = 3;
    	}
    	// [270..360] deg - 4th quadrant
    	else if(stepCount < 4*LOOK_UP_TABLE_SIZE)
    	{
            SetCCPVal(nindx, indx);
    	   	comm_state = 4;
    	}		
    }
    else
    {
    	if(CCPValReady)
    	{
    	    SetCCPVal(EEDATL, EEDATL);
    	    CCPValReady = 0;
    	 }
	 }
}


void StepperInit(void)
{
    // Initialize ECCP 1&2 for full bridge PWM
	CCP1CON = CCP1CON_INIT;
 	CCP2CON = CCP1CON_INIT;
 	// alternate pin functions for some PWM outputs
 	APFCON = APFCON_INIT;
   	comm_state = 1;
   	// Select Timer2 for ECCP 1&2 PWM
   	CCPTMRS0 = 0x00;
   	// Select special event compare mode for CCP4 which will control interrupt rate
   	CCP4CON = 0b00001011; 
    // Longest period for initial interrupt
   	CCPR4H = 0xFF;
   	CCPR4L = 0xFF;
	stepCount = 0;
    // Steering bits off, Sync enabled
    PSTR1CON = 0b00010000;
    PSTR2CON = 0b00010000;
  	GIE = 1;
	PEIE = 1;
	// enable CCP4 compare interrupts
	CCP4IE = 1;
	run_mode = RUN;
	startup_complete_flag = 1;
}