// SelfTest.c
//
// Change History:
// Author      Date        Comment
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// w.r.brown   2012.04.05  Changed temperature self-test lower limit from 18 to 5 deg C.
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
// F1 LV Evaluation board self test.
//
// The self test loop is entered when board auto-detect senses the self-test loop-back board.
//
// Hardware checks:
//
//      Test                      Covered by
//      ----                      -----------
//   1. Loop-back connections     SELF 
//   2. Temperature sensor        SELF
//   3. R4 potentiometer          POT
//   4. All LCD segments          LCD
//   5. LEDs D1, D2, D3, D4       All
//   6. Pushbuttons SW1 and SW2   Test advance, and reset
//   7. Timer 1 oscillator.       All (part of LED testing)
//   8. Battery voltage boost.    BATTERY (with External supply and voltmeter)
//   9. USB to fast serial        External PC
//
#include <xc.h>
#include "GenericTypeDefs.h"
#include "Main.h"
#include "UNIO_support.h"
#include "lcd2.h"
#include "i2c.h"
#include "input.h"
#include "Monitor.h"
#include "rtcc.h"
#include "mcp9800.h"

enum {SELF, POT, BATTERY, LCD, LCD_OFF} test_mode;

extern bit         TMR0_button_flag;
extern bit         stop_flag;
extern bit         run_flag;
extern mode_lcd    lcd_mode;
extern int         pot_value;
extern event_t     btn_event;

////////////////////////////////////////////////////////
// define all LCD segments used
#define LCD0	0b10001011
#define LCD1	0b00001111
#define LCD2	0b00001100
#define LCD3	0b01100000
#define LCD4	0b00000110
#define LCD5	0b00011000

////////////////////////////////////////////////////////
// prototypes
void self_test(void);
void SelfTestTasks(void);
void AcquireTestData(void);
void UpdateTestDisplay(void);
void ShowAllSegments(void);
void ClearAllSegments(void);
void BlinkLEDs(void);
BOOL TestLoopBack(void);
BOOL TestAllBits(void);
void InitLoopBack(void);

/************************************************************************
*                                                                       *
*      Function:       self_test(void)                                  *
*                                                                       *
*      Description:    Production tests of F1 LV Evaluation board       *
*                                                                       *
*      Parameters:                                                      *
*      Return value:   None                                             *
*                                                                       *
*      Note:                                                            *
*                                                                       *
*************************************************************************/

void self_test(void)
{
	MonitorInit();
	InitLoopBack();
	test_mode = SELF;
	while(1)
	{
		ServiceCommunicator();
      	rtcc_handler();
      	BlinkLEDs();
      	TimeBaseManager();
		if(TMR0_button_flag)
		{
			TMR0_button_flag=0;
			SelfTestTasks();
			// sense rising WA edge
			while(WA);
			while(!WA);
		    UpdateTestDisplay();
		}
	}
}

void SelfTestTasks(void)
{
	if(input_event()==BUTTON_RELEASED) test_mode++;
	AcquireTestData();

}

void AcquireTestData(void)
{
	int tmp; 
    
    switch(test_mode)
    {
	    case SELF:
	        stop_flag = TestLoopBack();
    		tmp = mcp9800_get_temp();
    		// t is deg C * 10.
    		// t should be room temperature (i.e. 41 to 85 deg F)
    		run_flag = ((tmp < 50) || (tmp > 300));
	    	break;
	    case POT:
	        pot_value = get_adc(SPEED_CHANNEL);
	    	break;
	    case BATTERY:
			pot_value = get_adc(BATTERY_CHANNEL);	        
	    	break;
	    default:
	        // LCD and LCD_OFF do not require data acquisition
	    	break;
	}
}

void UpdateTestDisplay(void)
{
    switch(test_mode)
    {
	    default:
	    case SELF:
	    	F_1 = 1;
	        if(run_flag || stop_flag)
	        {
			    lcd_display_character(1,LETTER_F);
	            lcd_display_character(2,LETTER_A);
	            lcd_display_character(3,LETTER_I);
	            lcd_display_character(4,LETTER_L);
	            if(stop_flag) lcd_display_character(5,LETTER_L);	        
	            if(run_flag) lcd_display_character(6,LETTER_t);	        
		    }else{
			    lcd_display_character(1,LETTER_P);
	            lcd_display_character(2,LETTER_A);
	            lcd_display_character(3,LETTER_S);
	            lcd_display_character(4,LETTER_S);
	            lcd_display_character(5,BLANK);
	            lcd_display_character(6,BLANK);
		    }
		    break;
	    case POT:
	    	F_1 = 0;
	    	F_2 = 1;
        	display_int(pot_value);
	        lcd_display_character(5,BLANK);
	        lcd_display_character(6,BLANK);
   	    	break;
   	    case BATTERY:
	    	F_2 = 0;
	    	F_3 = 1;
        	display_int(pot_value);
        	break;
	    case LCD:
	        ShowAllSegments();
	    	break;
	    case LCD_OFF:
	        ClearAllSegments();
	        test_mode = SELF;
	    	break;
	}
}

void ShowAllSegments(void)
{
	// Segments 0-15
	// COM0
	LCDDATA0 = LCD0;
	LCDDATA1 = LCD1;
	LCDDATA2 = LCD2;
	// COM1
	LCDDATA3 = LCD0;
	LCDDATA4 = LCD1;
	LCDDATA5 = LCD2;
	// COM2
	LCDDATA6 = LCD0;
	LCDDATA7 = LCD1;
	LCDDATA8 = LCD2;
	// COM3
	LCDDATA9 = LCD0;
	LCDDATA10 = LCD1;
	LCDDATA11 = LCD2;
	// Segments 16-45
	// COM0
	LCDDATA12 = LCD3;
	LCDDATA13 = LCD4;
	LCDDATA14 = LCD5;
	// COM1
	LCDDATA15 = LCD3;
	LCDDATA16 = LCD4;
	LCDDATA17 = LCD5;
	// COM2
	LCDDATA18 = LCD3;
	LCDDATA19 = LCD4;
	LCDDATA20 = LCD5;
	// COM3
	LCDDATA21 = LCD3;
	LCDDATA22 = LCD4;
	LCDDATA23 = LCD5;
}

void ClearAllSegments(void)
{
	// Segments 0-15
	// COM0
	LCDDATA0 = 0x00;
	LCDDATA1 = 0x00;
	LCDDATA2 = 0x00;
	// COM1
	LCDDATA3 = 0x00;
	LCDDATA4 = 0x00;
	LCDDATA5 = 0x00;
	// COM2
	LCDDATA6 = 0x00;
	LCDDATA7 = 0x00;
	LCDDATA8 = 0x00;
	// COM3
	LCDDATA9 = 0x00;
	LCDDATA10 = 0x00;
	LCDDATA11 = 0x00;
	// Segments 16-45
	// COM0
	LCDDATA12 = 0x00;
	LCDDATA13 = 0x00;
	LCDDATA14 = 0x00;
	// COM1
	LCDDATA15 = 0x00;
	LCDDATA16 = 0x00;
	LCDDATA17 = 0x00;
	// COM2
	LCDDATA18 = 0x00;
	LCDDATA19 = 0x00;
	LCDDATA20 = 0x00;
	// COM3
	LCDDATA21 = 0x00;
	LCDDATA22 = 0x00;
	LCDDATA23 = 0x00;
}

void BlinkLEDs(void)
{
	// call this routine repeatedly to
	// cycle through all LEDs flashing each on for 1 second
	
    time_t t;
	
    time(&t);
        
    switch (t % 4)
    {
        case 0:
        	LATA4 = 0;
        	LATA6 = 1;
        	break;
        case 1:
        	LATA6 = 0;
            LATA7 = 1;
        	break;
        case 2:
            LATA7 = 0;
        	LATC5 = 1;
        	break;
        default:
        case 3:
        	LATC5 = 0;
        	LATA4 = 1;
        	break;
    }
}

BOOL TestLoopBack(void)
{
	BOOL TestFail;
	// verify all wiring to J3 through the self-test loop-back fixture

	// clear all output bits
	LATC = 0x00;
	LATD = 0x00;
	LATE = 0x00;
	LATF = 0x00;
	LATG = 0x00;
	
	// pre-clear the test failure bit. Any failure will set the flag.
	TestFail = 0;
	
	// Test one bit at a time and ensure all other bits don't respond
	LATC2 = 1;
	if(TestAllBits()) TestFail = 1;
	LATC2 = 0;
	LATD4 = 1;
	if(TestAllBits()) TestFail = 1;
	LATD4 = 0;
	LATD6 = 1;
	if(TestAllBits()) TestFail = 1;
	LATD6 = 0;
	LATE7 = 1;
	if(TestAllBits()) TestFail = 1;
	LATE7 = 0;
	LATF3 = 1;
	if(TestAllBits()) TestFail = 1;
	LATF3 = 0;
	LATF5 = 1;
	if(TestAllBits()) TestFail = 1;
	LATF5 = 0;
	LATF7 = 1;
	if(TestAllBits()) TestFail = 1;
	LATF7 = 0;
	LATG0 = 1;
	if(TestAllBits()) TestFail = 1;
	LATG0 = 0;
	
	return TestFail;
}

BOOL TestAllBits(void)
{
	// When the latch bit is set then verify that the corresponding loop-back bit reads one
	// and all other loop-back bits read zero.
	
	BOOL TestFail;
	
	// preset fail flag to zero;
	TestFail = 0;
	
	// RC2 -> RG4
	if(LATC2 == 0)
	{
		if(RG4 == 1)
			TestFail = 1;
	}else{
		if(RG4 == 0)
			TestFail = 1;
	}
	//RD6 -> RD5
	if(LATD6 == 0)
	{
		if(RD5 == 1)
			TestFail = 1;
	}else{
		if(RD5 == 0)
			TestFail = 1;
	}
	//RD4 -> RE0
	if(LATD4 == 0)
	{
		if(RE0 == 1)
			TestFail = 1;
	}else{
		if(RE0 == 0)
			TestFail = 1;
	}
	//RE7 -> RE2
	if(LATE7 == 0)
	{
		if(RE2 == 1)
			TestFail = 1;
	}else{
		if(RE2 == 0)
			TestFail = 1;
	}
	//RF3 -> RF4
	if(LATF3 == 0)
	{
		if(RF4 == 1)
			TestFail = 1;
	}else{
		if(RF4 == 0)
			TestFail = 1;
	}
	//RF5 -> RF0
	if(LATF5 == 0)
	{
		if(RF0 == 1)
			TestFail = 1;
	}else{
		if(RF0 == 0)
			TestFail = 1;
	}
	//RF7 -> RF6
	if(LATF7 == 0)
	{
		if(RF6 == 1)
			TestFail = 1;
	}else{
		if(RF6 == 0)
			TestFail = 1;
	}
	//RG0 -> RE1
	if(LATG0 == 0)
	{
		if(RE1 == 1)
			TestFail = 1;
	}else{
		if(RE1 == 0)
			TestFail = 1;
	}
	
	return TestFail;
}

void InitLoopBack(void)
{
	// Loop-back connections
	//  1 -  2 : RC2 - RG4
	//  3 -  4 : RD6 - RD5
	//  5 -  7 : RE7 - RE2
	//  9 - 10 : RG0 - RE1
	// 11 - 12 : RD4 - RE0
	// 15 - 16 : RF7 - RF6
	// 17 - 18 : RF3 - RF4
	// 19 - 20 : RF5 - RF0
	
	// Set half of the loop-backs as outputs and the corresponding
	// connections as inputs.
	TRISC &= 0b11111011;
	TRISD &= 0b10101111;
	TRISE &= 0b01111111;
	TRISF &= 0b01010111;
	TRISG &= 0b11111110;
	
	TRISD |= 0b00100000;
	TRISE |= 0b00000111;
	TRISF |= 0b01010001;
	TRISG |= 0b00010000;

	ANSELF &= 0b10101110;
	
}